-- FICHERO PARA CREACION DE TABLAS: 2_CREACION_TABLAS.sql
-- Autor: David Giraldez Sanchez
-- Tutor: Jordi Ferrer Duran

-- NOTA: Acentos eliminados intencionadamente para facilitar la lectura con cualquier configuracion del cliente.

-- TODOS LOS COMANDOS DE ESTE SCRIPT DEBERAN LANZARSE COMO USUARIO TFG, O EL USUARIO QUE QUERAMOS UTILIZAR COMO ESQUEMA PRINCIPAL DE LA APLICACION
-- AL LANZAR ESTE SCRIPT COMO USUARIO TFG, LAS TABLAS SE CREAN EN SU TABLESPACE POR DEFECTO (SI NO SE INDICA LO CONTRARIO) Y SON DE SU PROPIEDAD.

-- COMENZARIAMOS POR ELIMINAR LAS TABLAS Y PAQUETES, POR SI EXISTEN; SI HEMOS CREADO EL ESQUEMA CON EL SCRIPT 1-Tablespace_y_Esquema.sql, SE HABRIAN ELIMINADO YA
/* 
DROP TABLE accion CASCADE CONSTRAINTS;
DROP TABLE analisis CASCADE CONSTRAINTS;
DROP TABLE asignadasesion CASCADE CONSTRAINTS;
DROP TABLE auditoria CASCADE CONSTRAINTS;
DROP TABLE departamento CASCADE CONSTRAINTS;
DROP TABLE empleado CASCADE CONSTRAINTS;
DROP TABLE estadistica CASCADE CONSTRAINTS;
DROP TABLE estadoaccion CASCADE CONSTRAINTS;
DROP TABLE estadovuln CASCADE CONSTRAINTS;
DROP TABLE estudioproceso CASCADE CONSTRAINTS;
DROP TABLE incumplimiento CASCADE CONSTRAINTS;
DROP TABLE incumplimientosanyodepto CASCADE CONSTRAINTS;
DROP TABLE incumplimientosanyoempleado CASCADE CONSTRAINTS;
DROP TABLE logoperaciones CASCADE CONSTRAINTS;
DROP TABLE muestreo CASCADE CONSTRAINTS;
DROP TABLE politica CASCADE CONSTRAINTS;
DROP TABLE proceso CASCADE CONSTRAINTS;
DROP TABLE sesion CASCADE CONSTRAINTS;
DROP TABLE vulnerabilidad CASCADE CONSTRAINTS;
*/

-- CREAMOS LAS TABLAS DEL SISTEMA (16): ACCION, ANALISIS, ASIGNADASESION, AUDITORIA, DEPARTAMENTO, EMPLEADO, ESTADOACCION, ESTADOVULN, ESTUDIOPROCESO, INCUMPLIMIENTO, LOGOPERACIONES, MUESTREO, POLITICA, PROCESO, SESION, VULNERABILIDAD

-- TAMBIEN CREAMOS LAS TABLAS QUE UTILIZAREMOS PARA EL MODULO ESTADISTICO (3): ESTADISTICA, INCUMPLIMIENTOSANYODEPTO, INCUMPLIMIENTOSANYOEMPLEADO

-- FINALMENTE, CREAMOS LAS SECUENCIAS Y LOS TRIGGERS QUE UTILIZAREMOS PARA RELLENAR LOS CAMPOS ID ("AUTOINCREMENTALES")

CREATE TABLE accion (
    id_accion          NUMBER(10) NOT NULL,
    nombre_accion      NVARCHAR2(50) NOT NULL,
    f_implantacion_acc DATE NOT NULL,
    f_estado_acc       DATE NOT NULL,
    url_accion         NVARCHAR2(100),
    id_vulnerabilidad  NUMBER(10) NOT NULL,
    id_estado_acc      NUMBER(10) NOT NULL,
    responsable        NUMBER(10) NOT NULL
);

COMMENT ON COLUMN accion.id_accion IS
    'Identificador nico de la accion.';

COMMENT ON COLUMN accion.f_estado_acc IS
    'Fecha en la que se ha actualizado el estado de la accion.';

ALTER TABLE accion ADD CONSTRAINT accion__pk PRIMARY KEY ( id_accion );

CREATE TABLE analisis (
    id_analisis       NUMBER(10) NOT NULL,
    nombre_analisis   NVARCHAR2(50) NOT NULL,
    tipo_analisis     NVARCHAR2(7) NOT NULL,
    nombre_equipo     NVARCHAR2(50) NOT NULL,
    f_inicio_analisis DATE NOT NULL,
    f_fin_analisis    DATE,
    url_analisis      NVARCHAR2(100)
);

ALTER TABLE analisis
    ADD CONSTRAINT tipos_de_analisis CHECK ( tipo_analisis IN ( 'EXTERNO', 'INTERNO' ) );

COMMENT ON COLUMN analisis.id_analisis IS
    'Identificador nico del anlisis.';

COMMENT ON COLUMN analisis.tipo_analisis IS
    'Aceptar los valores EXTERNO o INTERNO';

COMMENT ON COLUMN analisis.nombre_equipo IS
    'Nombre del equipo que realiza el anlisis.';

COMMENT ON COLUMN analisis.f_inicio_analisis IS
    'Fecha de inicio del anlisis';

COMMENT ON COLUMN analisis.f_fin_analisis IS
    'Fecha de finalizacin del anlisis';

COMMENT ON COLUMN analisis.url_analisis IS
    'URL de la documentacin relativa al anlisis en el gestor documental de la empresa';

ALTER TABLE analisis ADD CONSTRAINT analisis_pk PRIMARY KEY ( id_analisis );

CREATE TABLE asignadasesion (
    sesion_id_sesion     NUMBER(10) NOT NULL,
    empleado_id_empleado NUMBER(10) NOT NULL,
    ha_finalizado_sesion NUMBER(1) NOT NULL,
    f_fin_sesion         DATE
);

ALTER TABLE asignadasesion
    ADD CHECK ( ha_finalizado_sesion IN ( 0, 1 ) );

ALTER TABLE asignadasesion ADD CONSTRAINT asignadasesion_pk PRIMARY KEY ( sesion_id_sesion, empleado_id_empleado );

CREATE TABLE auditoria (
    id_auditoria       NUMBER(10) NOT NULL,
    nombre_auditoria   NVARCHAR2(50) NOT NULL,
    tipo_auditoria     NVARCHAR2(10) NOT NULL,
    f_inicio_auditoria DATE NOT NULL,
    f_fin_auditoria    DATE
);

COMMENT ON COLUMN auditoria.id_auditoria IS
    'Identificador nico de la auditora.';

COMMENT ON COLUMN auditoria.nombre_auditoria IS
    'Nombre los ms  descriptivo posible que asignaremos ala auditoria.';

COMMENT ON COLUMN auditoria.tipo_auditoria IS
    'Aceptara los valores EXTERNA o INTERNA';

COMMENT ON COLUMN auditoria.f_fin_auditoria IS
    'Se registrar la fecha en que una auditora se da por finalizada';

ALTER TABLE auditoria ADD CONSTRAINT auditoria_pk PRIMARY KEY ( id_auditoria );

CREATE TABLE departamento (
    id_departamento           NUMBER(10) NOT NULL,
    nombre_depto              NVARCHAR2(50) NOT NULL,
    incumplimientos_empleados NUMBER(5) DEFAULT 0 NOT NULL,
    porc_empls_con_ses_pend   NUMBER(4, 2) DEFAULT 0 NOT NULL
);

COMMENT ON COLUMN departamento.id_departamento IS
    'Identificador nico del departamento.';

COMMENT ON COLUMN departamento.incumplimientos_empleados IS
    'Numero de incumplimientos de los empleados del departamento. Por defecto empieza en 0, y se actualizar de forma automtica al registrarse incumplimientos en el sistema.';

COMMENT ON COLUMN departamento.porc_empls_con_ses_pend IS
    'Porcentaje de los empleados del departamento que tienen sesiones de formacin pendientes. Valor por defecto 0, se ir actualizando de forma automtica con cada asignacin de sesin y con cada finalizacin de seion.';

ALTER TABLE departamento ADD CONSTRAINT departamento_pk PRIMARY KEY ( id_departamento );

CREATE TABLE empleado (
    id_empleado           NUMBER(10) NOT NULL,
    dni_empleado          NVARCHAR2(9) NOT NULL,
    nombre_emp            NVARCHAR2(25) NOT NULL,
    apellido1_emp         NVARCHAR2(50) NOT NULL,
    apellido2_emp         NVARCHAR2(50) NOT NULL,
    email_empleado        NVARCHAR2(50) NOT NULL,
    tfno_empleado         NUMBER(9) NOT NULL,
    es_empleado_en_activo NUMBER(1) NOT NULL,
    incumplimientos_emp   NUMBER(4) DEFAULT 0 NOT NULL,
    id_departamento       NUMBER(10) NOT NULL
);

ALTER TABLE empleado
    ADD CHECK ( es_empleado_en_activo IN ( 0, 1 ) );

COMMENT ON COLUMN empleado.id_empleado IS
    'Identificador nico del empleado en la empresa.';

COMMENT ON COLUMN empleado.tfno_empleado IS
    'Telfono de contacto del empleado.';

COMMENT ON COLUMN empleado.es_empleado_en_activo IS
    'Se indica cuando un empleado sigue en activo en la empresa o la ha dejado. Valores admitidos: 0/1';

COMMENT ON COLUMN empleado.incumplimientos_emp IS
    'Nmeros de incumplimientos registrados cometidos por parte del empleado.';

ALTER TABLE empleado ADD CONSTRAINT empleado_pk PRIMARY KEY ( id_empleado );

CREATE TABLE estadistica (
    id_consulta          NUMBER(10) NOT NULL,
    descripcion_consulta NVARCHAR2(300) NOT NULL,
    resultado_consulta   NUMBER(12, 2) NOT NULL
);

COMMENT ON COLUMN estadistica.id_consulta IS
    'IDentificador de la consulta';

COMMENT ON COLUMN estadistica.descripcion_consulta IS
    'Descripcion de la consulta cuyo resultado vamos a almacenar';

COMMENT ON COLUMN estadistica.resultado_consulta IS
    'Resultado almacenado, que da respuesta a la consulta.';

ALTER TABLE estadistica ADD CONSTRAINT estadistica_pk PRIMARY KEY ( id_consulta );

CREATE TABLE estadoaccion (
    id_estado_acc NUMBER(10) NOT NULL,
    estado_accion NVARCHAR2(25) NOT NULL
);

COMMENT ON COLUMN estadoaccion.id_estado_acc IS
    'identificador de estado de accion de mitigacin';

COMMENT ON COLUMN estadoaccion.estado_accion IS
    'Los estados por los que puede pasar una accin de mitigacin son (podran aadirse ms estados con el tiempo, pero de momento son estos los que se indican en el enunciado):
1 Definida
2 En proceso
3 Acabada
4 En Revisin (para aquellas que no se pueda ejecutar por algn motivo)

';

ALTER TABLE estadoaccion ADD CONSTRAINT estadoaccion_pk PRIMARY KEY ( id_estado_acc );

CREATE TABLE estadovuln (
    id_estado_vuln NUMBER(10) NOT NULL,
    estado_vuln    NVARCHAR2(25) NOT NULL
);

ALTER TABLE estadovuln
    ADD CHECK ( estado_vuln IN ( 'Identificada', 'No Mitigada', 'Parcialmente Mitigada', 'Totalmente Mitigada' ) );

COMMENT ON COLUMN estadovuln.id_estado_vuln IS
    'identificador de estado de vulnerabilidad';

COMMENT ON COLUMN estadovuln.estado_vuln IS
    'Los estados por los que puede pasar una vulnerabilidad son los siguientes (podran aadirse ms estados con el tiempo): 
1 Identificada
2 No Mitigada 
3 Parcialmente Mitigada (si hay al menos una accin de mitigacin asociada a la vulnerabilidad que est en estado acabada)
4 Totalmente Mitigada (si todas las acciones de mitigacin asociadas a la vulnerabilidad estn acabadas)
';

ALTER TABLE estadovuln ADD CONSTRAINT estadovuln_pk PRIMARY KEY ( id_estado_vuln );

CREATE TABLE estudioproceso (
    id_analisis      NUMBER(10) NOT NULL,
    id_proceso       NUMBER(10) NOT NULL,
    f_estudioproceso DATE NOT NULL
);

ALTER TABLE estudioproceso ADD CONSTRAINT estudioproceso_pk PRIMARY KEY ( id_analisis, id_proceso );

CREATE TABLE incumplimiento (
    id_incumplimiento          NUMBER(10) NOT NULL,
    f_deteccion_incumplimiento DATE NOT NULL,
    id_auditoria               NUMBER(10) NOT NULL,
    id_empleado                NUMBER(10) NOT NULL,
    id_politica                NUMBER(10) NOT NULL
);

COMMENT ON COLUMN incumplimiento.id_incumplimiento IS
    'Identificador nico del incumplimiento.';

ALTER TABLE incumplimiento ADD CONSTRAINT incumplimiento_pk PRIMARY KEY ( id_incumplimiento );

CREATE TABLE incumplimientosanyodepto (
    anyo                NUMBER(4) NOT NULL,
    id_departamento     NUMBER(10) NOT NULL,
    num_incumplimientos NUMBER(5) NOT NULL
);

COMMENT ON COLUMN incumplimientosanyodepto.anyo IS
    'Ao de los incumplimientos registrados.';

COMMENT ON COLUMN incumplimientosanyodepto.num_incumplimientos IS
    'Numero de incumplimientos anotados al departamento en el ao indicado.';

ALTER TABLE incumplimientosanyodepto ADD CONSTRAINT incumplimientosanyodepto_pk PRIMARY KEY ( anyo, id_departamento );

CREATE TABLE incumplimientosanyoempleado (
    id_empleado                  NUMBER(10) NOT NULL,
    anyo                         INTEGER NOT NULL,
    incumplimientos_propios      NUMBER(4) NOT NULL,
    incumplimientos_departamento NUMBER(4) NOT NULL,
    incumplimientos_totales      NUMBER(5) NOT NULL
);

COMMENT ON COLUMN incumplimientosanyoempleado.id_empleado IS
    'Identificador nico de empleado.';

COMMENT ON COLUMN incumplimientosanyoempleado.anyo IS
    'Ao para el que se estn anotando los incumplimientos del empleado.';

COMMENT ON COLUMN incumplimientosanyoempleado.incumplimientos_propios IS
    'Incumplimientos cometidos por el empleado en el ao en cuestin.';

COMMENT ON COLUMN incumplimientosanyoempleado.incumplimientos_departamento IS
    'Incumplimientos cometidos por el departamento al que pertenece el empleado en el ao en cuestin.';

COMMENT ON COLUMN incumplimientosanyoempleado.incumplimientos_totales IS
    'Suma de los incumplimientos propios y del departamento en el ao en cuestin.';

ALTER TABLE incumplimientosanyoempleado ADD CONSTRAINT usuariosconincumplimientos_pk PRIMARY KEY ( anyo, id_empleado );

CREATE TABLE logoperaciones (
    id_log               NUMBER(10) NOT NULL,
    f_ejecucion          DATE NOT NULL,
    usuario              NVARCHAR2(15) NOT NULL,
    nombre_procedimiento NVARCHAR2(100) NOT NULL,
    params_entrada       NVARCHAR2(300) NOT NULL,
    resultado            NVARCHAR2(300) NOT NULL
);

COMMENT ON COLUMN logoperaciones.id_log IS
    'Identificador de los registros de log.';

COMMENT ON COLUMN logoperaciones.f_ejecucion IS
    'Fecha y hora de ejecucin del procedimiento.';

COMMENT ON COLUMN logoperaciones.usuario IS
    'Usuario de BBDD que ha ejecutado el procedimiento.';

COMMENT ON COLUMN logoperaciones.nombre_procedimiento IS
    'Nombre del procedimiento ejecutado.';

COMMENT ON COLUMN logoperaciones.params_entrada IS
    'Parmetros de entrada del procedimiento.';

COMMENT ON COLUMN logoperaciones.resultado IS
    'Resultado de la ejecucin del procedimiento.';

ALTER TABLE logoperaciones ADD CONSTRAINT logoperaciones_pk PRIMARY KEY ( id_log );

CREATE TABLE muestreo (
    id_politica  NUMBER(10) NOT NULL,
    id_auditoria NUMBER(10) NOT NULL,
    id_empleado  NUMBER(10) NOT NULL,
    f_muestreo   DATE NOT NULL
);

ALTER TABLE muestreo
    ADD CONSTRAINT muestreo_pk PRIMARY KEY ( id_politica, id_auditoria, id_empleado );

CREATE TABLE politica (
    id_politica         NUMBER(10) NOT NULL,
    nombre_politica     NVARCHAR2(50) NOT NULL,
    url_politica        NVARCHAR2(100),
    num_incumplimientos NUMBER(10) DEFAULT 0 NOT NULL
);

COMMENT ON COLUMN politica.id_politica IS
    'Identificador nico de la poltica.';

COMMENT ON COLUMN politica.num_incumplimientos IS
    'Numero total de incumplimientos de la poltica. Por defecto es 0.Se actualizar automticamente cuando se registre un incumplimiento.';

ALTER TABLE politica ADD CONSTRAINT politica_pk PRIMARY KEY ( id_politica );

CREATE TABLE proceso (
    id_proceso                NUMBER(10) NOT NULL,
    nombre_proceso            NVARCHAR2(50) NOT NULL,
    url_proceso               NVARCHAR2(100),
    ha_sido_analizado_proceso NUMBER(1) DEFAULT 0 NOT NULL,
    num_vuln_detectadas       NUMBER(4) DEFAULT 0 NOT NULL
);

ALTER TABLE proceso
    ADD CHECK ( ha_sido_analizado_proceso IN ( 0, 1 ) );

COMMENT ON COLUMN proceso.id_proceso IS
    'Identificador nico del proceso.';

COMMENT ON COLUMN proceso.url_proceso IS
    'URL de la documentacin relativa al proceso en el gestor documental de la empresa';

COMMENT ON COLUMN proceso.ha_sido_analizado_proceso IS
    'Se registrar si el proceso ha sido analizado en alguna ocasin por algn anlisis o no.Valores admitidos: 0/1';

COMMENT ON COLUMN proceso.num_vuln_detectadas IS
    'Numero de vulnerabilidades detectadas asociadas al proceso. Por defecto es 0. Se actualizar automticamente al registrar una vulnerabilidad.';

ALTER TABLE proceso ADD CONSTRAINT proceso_pk PRIMARY KEY ( id_proceso );

CREATE TABLE sesion (
    id_sesion        NUMBER(10) NOT NULL,
    titulo_sesion    NVARCHAR2(100) NOT NULL,
    modalidad_sesion NVARCHAR2(15) NOT NULL,
    f_inicio_sesion  DATE NOT NULL
);

ALTER TABLE sesion
    ADD CHECK ( modalidad_sesion IN ( 'PRESENCIAL', 'TELEMATICA' ) );

COMMENT ON COLUMN sesion.id_sesion IS
    'Identificador ncio de la sesin de formacin.';

COMMENT ON COLUMN sesion.modalidad_sesion IS
    'Tipos de sesiones de formacin, podrn ser presenciales o telematicas.';

ALTER TABLE sesion ADD CONSTRAINT sesion_pk PRIMARY KEY ( id_sesion );

CREATE TABLE vulnerabilidad (
    id_vulnerabilidad NUMBER(10) NOT NULL,
    id_analisis       NUMBER(10) NOT NULL,
    id_proceso        NUMBER(10) NOT NULL,
    f_deteccion_vuln  DATE NOT NULL,
    id_estado_vuln    NUMBER(10) NOT NULL,
    es_vuln_critica   NUMBER(1) DEFAULT 0 NOT NULL,
    nombre_vuln       NVARCHAR2(50) NOT NULL,
    url_vuln          VARCHAR2(100)
);

ALTER TABLE vulnerabilidad
    ADD CHECK ( es_vuln_critica IN ( 0, 1 ) );

COMMENT ON COLUMN vulnerabilidad.id_vulnerabilidad IS
    'Identificador nico de una vulnerabilidad';

COMMENT ON COLUMN vulnerabilidad.id_analisis IS
    'Identificador nico del anlisis en el que se ha descubierto la vulnerabilidad.';

COMMENT ON COLUMN vulnerabilidad.id_proceso IS
    'Identificador nico del proceso al que afecta la vulnerabilidad que se ha descubierto.';

COMMENT ON COLUMN vulnerabilidad.f_deteccion_vuln IS
    'Fecha en la que se detecta la vulnerabilidad.';

COMMENT ON COLUMN vulnerabilidad.id_estado_vuln IS
    'Identificador del estado en el que se encuentra la vulnerabilidad. Los estados posibles se han definido en el enunciado, pero pueden cambiar.';

COMMENT ON COLUMN vulnerabilidad.es_vuln_critica IS
    'Indica si se ha considerado la vulnerabilidad como crtica o no. Valores admitidos: 0/1';

COMMENT ON COLUMN vulnerabilidad.nombre_vuln IS
    'Nombre asignado a la vulnerabilidad.';

COMMENT ON COLUMN vulnerabilidad.url_vuln IS
    'URL de la documentacin relativa a la vulnerabilidad en el gestor documental de la empresa.';

ALTER TABLE vulnerabilidad ADD CONSTRAINT vulnerabilidad_pk PRIMARY KEY ( id_vulnerabilidad );

ALTER TABLE vulnerabilidad
    ADD CONSTRAINT analisis_vulnerabilidad FOREIGN KEY ( id_analisis )
        REFERENCES analisis ( id_analisis );

ALTER TABLE asignadasesion
    ADD CONSTRAINT asignadasesion_empleado_fk FOREIGN KEY ( empleado_id_empleado )
        REFERENCES empleado ( id_empleado );

ALTER TABLE asignadasesion
    ADD CONSTRAINT asignadasesion_sesion_fk FOREIGN KEY ( sesion_id_sesion )
        REFERENCES sesion ( id_sesion );

ALTER TABLE incumplimiento
    ADD CONSTRAINT auditoria_incumplimiento FOREIGN KEY ( id_auditoria )
        REFERENCES auditoria ( id_auditoria );

ALTER TABLE muestreo
    ADD CONSTRAINT auditoria_muestreo FOREIGN KEY ( id_auditoria )
        REFERENCES auditoria ( id_auditoria );

ALTER TABLE empleado
    ADD CONSTRAINT empleado_departamento FOREIGN KEY ( id_departamento )
        REFERENCES departamento ( id_departamento );

ALTER TABLE incumplimiento
    ADD CONSTRAINT empleado_incumplimiento FOREIGN KEY ( id_empleado )
        REFERENCES empleado ( id_empleado );

ALTER TABLE muestreo
    ADD CONSTRAINT empleado_muestreo FOREIGN KEY ( id_empleado )
        REFERENCES empleado ( id_empleado );

ALTER TABLE accion
    ADD CONSTRAINT estadoaccion_accion FOREIGN KEY ( id_estado_acc )
        REFERENCES estadoaccion ( id_estado_acc );

ALTER TABLE vulnerabilidad
    ADD CONSTRAINT estadovuln_vulnerabilidad FOREIGN KEY ( id_estado_vuln )
        REFERENCES estadovuln ( id_estado_vuln );

ALTER TABLE estudioproceso
    ADD CONSTRAINT estudioproceso_analisis FOREIGN KEY ( id_analisis )
        REFERENCES analisis ( id_analisis );

ALTER TABLE estudioproceso
    ADD CONSTRAINT estudioproceso_proceso FOREIGN KEY ( id_proceso )
        REFERENCES proceso ( id_proceso );

ALTER TABLE incumplimientosanyodepto
    ADD CONSTRAINT incanyodepto_empleado FOREIGN KEY ( id_departamento )
        REFERENCES departamento ( id_departamento );

ALTER TABLE incumplimientosanyoempleado
    ADD CONSTRAINT incanyoempl_empleado FOREIGN KEY ( id_empleado )
        REFERENCES empleado ( id_empleado );

ALTER TABLE incumplimiento
    ADD CONSTRAINT politica_incumplimiento FOREIGN KEY ( id_politica )
        REFERENCES politica ( id_politica );

ALTER TABLE muestreo
    ADD CONSTRAINT politica_muestreo FOREIGN KEY ( id_politica )
        REFERENCES politica ( id_politica );

ALTER TABLE vulnerabilidad
    ADD CONSTRAINT proceso_vulnerabilidad FOREIGN KEY ( id_proceso )
        REFERENCES proceso ( id_proceso );

ALTER TABLE accion
    ADD CONSTRAINT responsable_accion FOREIGN KEY ( responsable )
        REFERENCES empleado ( id_empleado );

ALTER TABLE accion
    ADD CONSTRAINT vulnerabilidad_accion FOREIGN KEY ( id_vulnerabilidad )
        REFERENCES vulnerabilidad ( id_vulnerabilidad );