CREATE OR REPLACE PACKAGE BODY PKG_FUNCIONAMIENTO AS 

/* -- FICHERO PARA CREACION DE CUERPO DE PAQUETE: Cuerpo del paquete para los procedimientos de funcionamiento del sistema: 
 * Actualizacion de campos calculados en las tablas del sistema, actualizacion de los campos de la tabla estadistica
 * Autor: David Giraldez Sanchez
 * Tutor: Jordi Ferrer Duran
 */ 
      
-- NOTA: Acentos eliminados intencionadamente para facilitar la lectura con cualquier configuracion del cliente.

-- ESTE SCRIPT DEBERA LANZARSE COMO USUARIO TFG, O EL USUARIO QUE QUERAMOS UTILIZAR COMO ESQUEMA PRINCIPAL DE LA APLICACION
-- AL LANZAR ESTE SCRIPT COMO USUARIO TFG, EL PAQUETE SE CREA EN SU ESQUEMA Y ES DE SU PROPIEDAD.   

/****************************************
**** ACTUALIZACION CAMPOS CALCULADOS ****
*****************************************/

PROCEDURE ACTUALIZAR_VULNERABILIDADES_DE_PROCESO(
	pProceso IN PROCESO.id_proceso%TYPE, 
	RSP OUT VARCHAR2) AS 

	MI_EXC   EXCEPTION;
	PAR_IN   VARCHAR2(1000);
	PROC     VARCHAR2(100);
	busca_pk	NUMBER(1);
	vNum	NUMBER(4);
	
BEGIN 
	PAR_IN := 'Proceso: '||pProceso;
	PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_VULNERABILIDADES_DE_PROCESO'; 

	-- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
	RSP := 'RESULTADO: ERROR: El identificador de proceso no puede ser nulo.';
	IF pProceso IS NULL THEN RAISE MI_EXC; END IF;
	
	-- Comprobamos que el id proporcionado identifica a un proceso en el sistema.
	-- Al ser PK, lo comprobaria el SGBD, pero asi personalizamos el mensaje en el log de operaciones en caso de error.	
	SELECT COUNT(1) INTO busca_pk FROM PROCESO WHERE id_proceso = pProceso;
	IF busca_pk = 0 THEN -- No existe
		RSP := 'RESULTADO: ERROR: El proceso indicado no existe.';
		RAISE MI_EXC; 
	ELSE -- Si existe, obtenemos la cantidad de vulnerabilidades detectadas asociadas al proceso en cuestion, y lo actualizamos.
		SELECT COUNT(1) INTO vNum FROM VULNERABILIDAD WHERE id_proceso = pProceso;
		UPDATE PROCESO SET num_vuln_detectadas = vNum WHERE id_proceso = pProceso;
		-- Registramos la actividad en el LOGOPERACIONES
		RSP := 'RESULTADO: OK';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
	END IF;

	-- ANOTAMOS LOS CAMBIOS
	COMMIT;
	
   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
	
   END ACTUALIZAR_VULNERABILIDADES_DE_PROCESO;
   
   PROCEDURE ACTUALIZAR_PORCENTAJE_DE_DEPARTAMENTO ( 
   pDepto IN DEPARTAMENTO.id_departamento%TYPE,
   RSP OUT VARCHAR2) AS 
   
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   vNum_total_empl_depto	NUMBER(10);
   vNum_empl_ses_pendientes	NUMBER(10);
   vPorcentaje	NUMBER(10);
      
  BEGIN
  -- Dado un determinado departamento de la empresa, y teniendo en cuenta el momento de ejecutar la consulta, porcentaje de usuarios del departamento que no han acabado todas las formaciones de seguridad asignadas.  
  PAR_IN := 'Actualizacion del resultado de la consulta estadistica numero 5.';
  PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_PORCENTAJE_DE_DEPARTAMENTO';
  -- NOTA: Con cada alta, baja o modificacion de asignacion de sesion, se actualiza el campo correspondiente en DEPARTAMENTO, y en este caso NO  se anota en la tabla estadística, se consultara desde la tabla de DEPARTAMENTO

  SELECT COUNT(1) INTO vNum_total_empl_depto FROM EMPLEADO WHERE id_departamento = pDepto;
  SELECT COUNT(e.id_empleado) INTO vNum_empl_ses_pendientes FROM EMPLEADO e, ASIGNADASESION a WHERE e.id_empleado = a.Empleado_id_empleado AND a.ha_finalizado_sesion = 0 AND e.id_departamento = pDepto; -- Consideramos 0 como FALSO, NO ha finalizado la sesion
  vPorcentaje := (vNum_empl_ses_pendientes/vNum_total_empl_depto)*100;
  
  UPDATE DEPARTAMENTO SET porc_empls_con_ses_pend = vPorcentaje;
  RSP := 'RESULTADO ACTUALIZACION PORCENTAJE DE EMPLEADOS CON SESIONES PENDIENTES: '||vPorcentaje||' - OK.';
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos  
  EXCEPTION
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;

END ACTUALIZAR_PORCENTAJE_DE_DEPARTAMENTO; 

PROCEDURE ACTUALIZAR_ESTADO_VULNERABILIDAD (
	pVulnerabilidad IN VULNERABILIDAD.id_vulnerabilidad%TYPE, 
	RSP OUT VARCHAR2) AS 

	MI_EXC   EXCEPTION;
	PAR_IN   VARCHAR2(1000);
	PROC     VARCHAR2(100);
	busca_pk	NUMBER(1);
	busca_accion	NUMBER(10);
	vEstado		NUMBER(1);
	
BEGIN 
	PAR_IN := 'Vulnerabilidad: '||pVulnerabilidad;
	PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_ESTADO_VULNERABILIDAD'; 

	-- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
	RSP := 'RESULTADO: ERROR: El identificador de vulnerabilidad no puede ser nulo.';
	IF pVulnerabilidad IS NULL THEN RAISE MI_EXC; END IF;
	
	-- Comprobamos que el id proporcionado identifica a una vulnerabilidad en el sistema.
	-- Al ser PK, lo comprobaria el SGBD, pero asi personalizamos el mensaje en el log de operaciones en caso de error.	
	SELECT COUNT(1) INTO busca_pk FROM VULNERABILIDAD WHERE id_vulnerabilidad = pVulnerabilidad;
	IF busca_pk = 0 THEN -- No existe
		RSP := 'RESULTADO: ERROR: La vulnerabilidad indicada no existe.';
		RAISE MI_EXC; 
	ELSE -- Si existe, comprobamos si existen acciones asociadas a la vulnerabilidades en cuestion que no esten acabadas, y actualizamos el estado.
		SELECT COUNT(1) INTO busca_accion FROM VULNERABILIDAD v, ACCION a WHERE a.id_vulnerabilidad = v.id_vulnerabilidad AND a.id_estado_acc <> 3;
		-- Si la vulnerabilidad NO tiene acciones asociadas que no esten en estado 3, marcamos la vulnerabilidad como TOTALMENTE MITIGADA (estado 4).
		IF busca_accion = 0 THEN 
			UPDATE VULNERABILIDAD SET id_estado_vuln = 4 WHERE id_vulnerabilidad = pVulnerabilidad;
			-- Registramos la actividad en el LOGOPERACIONES
			RSP := 'RESULTADO: OK';
			INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		END IF; -- Si tiene acciones asociadas no acabadas, no cambiamos el estado de la vulnerabilidad.
	END IF;

	-- ANOTAMOS LOS CAMBIOS
	COMMIT;
	
   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;

END ACTUALIZAR_ESTADO_VULNERABILIDAD;

PROCEDURE ACTUALIZAR_INCUMPLIMIENTOS_DE_POLITICA (
	pPolitica IN POLITICA.id_politica%TYPE, 
	RSP OUT VARCHAR2) AS 

	MI_EXC   EXCEPTION;
	PAR_IN   VARCHAR2(1000);
	PROC     VARCHAR2(100);
	busca_pk	NUMBER(1);
	vNum_incumplimientos	NUMBER(10);
	vPolitica		NUMBER(1);
	
BEGIN 
	PAR_IN := 'Politica: '||pPolitica;
	PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_INCUMPLIMIENTOS_DE_POLITICA'; 

	-- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
	RSP := 'RESULTADO: ERROR: El identificador de politica no puede ser nulo.';
	IF pPolitica IS NULL THEN RAISE MI_EXC; END IF;
	
	-- Comprobamos que el id proporcionado identifica a una politica en el sistema.
	-- Al ser PK, lo comprobaria el SGBD, pero asi personalizamos el mensaje en el log de operaciones en caso de error.	
	SELECT COUNT(1) INTO busca_pk FROM POLITICA WHERE id_politica = pPolitica;
	IF busca_pk = 0 THEN -- No existe
		RSP := 'RESULTADO: ERROR: La politica indicada no existe.';
		RAISE MI_EXC; 
	ELSE -- Si existe, comprobamos cuantos incumplimientos asociados a la politica en cuestion están registrados, y actualizamos el estado.
		SELECT COUNT(1) INTO vNum_incumplimientos FROM INCUMPLIMIENTO WHERE id_politica = pPolitica;
		UPDATE POLITICA SET num_incumplimientos = vNum_incumplimientos WHERE id_politica = pPolitica;
		-- Registramos la actividad en el LOGOPERACIONES
		RSP := 'RESULTADO: OK';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
	END IF;

	-- ANOTAMOS LOS CAMBIOS
	COMMIT;
	
   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;

END ACTUALIZAR_INCUMPLIMIENTOS_DE_POLITICA;




/**********************************
**** ACTUALIZACION ESTADISTICA ****
***********************************/

PROCEDURE ACTUALIZAR_CONSULTA_1 ( 
   RSP OUT VARCHAR2) AS 
   
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   vProceso	NUMBER(10);
   
  BEGIN
  -- Proceso de gestion interno que, teniendo en cuenta toda la informacion de que se dispone en la BD, ha tenido un mayor numero de vulnerabilidades detectadas.
  PAR_IN := 'Actualizacion del resultado de la consulta estadistica numero 1.';
  PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_CONSULTA_1';
  -- NOTA: Previamente habremos actualizado el campo num_vuln_detectadas de PROCESO mediante trigger.

  SELECT id_proceso INTO vProceso FROM (SELECT * FROM PROCESO ORDER BY num_vuln_detectadas DESC) WHERE ROWNUM = 1;
  UPDATE ESTADISTICA SET resultado_consulta = vProceso WHERE id_consulta = 1;
  RSP := 'RESULTADO ACTUALIZACION CONSULTA 1: '||vProceso||' - OK.';
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- Si ha saltado una excepcion, la procesamos  
  EXCEPTION
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  END ACTUALIZAR_CONSULTA_1;

PROCEDURE ACTUALIZAR_CONSULTA_2 ( 
   RSP OUT VARCHAR2) AS 
   
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   vTotal_vuln	NUMBER(10);
   vTotal_vuln_tot_mit	NUMBER(10);
   vPorcentaje	NUMBER(4,2);
   
  BEGIN
  -- Porcentaje de vulnerabilidades que, en el momento de ejecutar la consulta, están totalmente mitigadas.
  PAR_IN := 'Actualizacion del resultado de la consulta estadistica numero 2.';
  PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_CONSULTA_2';

  SELECT COUNT(1) INTO vTotal_vuln FROM VULNERABILIDAD;
  SELECT COUNT(1) INTO vTotal_vuln_tot_mit FROM VULNERABILIDAD WHERE id_estado_vuln = 4;
  vPorcentaje := ((vTotal_vuln_tot_mit/vTotal_vuln)*100);
  
  UPDATE ESTADISTICA SET resultado_consulta = vPorcentaje WHERE id_consulta = 2;
  RSP := 'RESULTADO ACTUALIZACION CONSULTA 2: '||vPorcentaje||' - OK.';
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- Si ha saltado una excepcion, la procesamos  
  EXCEPTION
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		
END ACTUALIZAR_CONSULTA_2; 

PROCEDURE ACTUALIZAR_CONSULTA_3 ( 
   RSP OUT VARCHAR2) AS 
   
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   vTotal_acc_no_acabadas	NUMBER(10);
      
  BEGIN
  -- Número total de acciones de mitigación que, en el momento de ejecutar la consulta, no están totalmente acabadas.
  PAR_IN := 'Actualizacion del resultado de la consulta estadistica numero 3.';
  PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_CONSULTA_3';

  SELECT COUNT(1) INTO vTotal_acc_no_acabadas FROM ACCION WHERE id_estado_acc <> 3;
  
  UPDATE ESTADISTICA SET resultado_consulta = vTotal_acc_no_acabadas WHERE id_consulta = 3;
  RSP := 'RESULTADO ACTUALIZACION CONSULTA 3: '||vTotal_acc_no_acabadas||' - OK.';
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- Si ha saltado una excepcion, la procesamos  
  EXCEPTION
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);

END ACTUALIZAR_CONSULTA_3;		

PROCEDURE ACTUALIZAR_CONSULTA_4 ( 
   RSP OUT VARCHAR2) AS 
   
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   vPolitica	NUMBER(10);
      
  BEGIN
  -- Política de seguridad que, en el momento de ejecutar la consulta, ha tenido más incumplimientos (teniendo en cuenta todos los departamentos de la empresa).   
  PAR_IN := 'Actualizacion del resultado de la consulta estadistica numero 4.';
  PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_CONSULTA_4';
  -- NOTA: Previamente habremos actualizado el campo num_incumplimientos de POLITICA mediante trigger.

  SELECT id_politica INTO vPolitica FROM (SELECT * FROM POLITICA ORDER BY num_incumplimientos DESC) WHERE ROWNUM = 1;
  UPDATE ESTADISTICA SET resultado_consulta = vPolitica WHERE id_consulta = 4;
  RSP := 'RESULTADO ACTUALIZACION CONSULTA 4: '||vPolitica||' - OK.';
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- Si ha saltado una excepcion, la procesamos  
  EXCEPTION
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);

END ACTUALIZAR_CONSULTA_4; 

-- El resultado de la CONSULTA 5 se obtiene mirando directamente la tabla DEPARTAMENTO, no la registramos en la tabla ESTADISTICA.
-- Dejamos el "hueco" por coherencia con el modelo que hemos definido.
/* PROCEDURE ACTUALIZAR_CONSULTA_5 ( 
   RSP OUT VARCHAR2);
*/

PROCEDURE ACTUALIZAR_CONSULTA_6 ( 
   RSP OUT VARCHAR2) AS 
   
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   vAnyo	NUMBER(4);
      
  BEGIN
  -- Teniendo en cuenta todas las auditorías externas realizadas, año en el cual se han detectado más incumplimientos (teniendo en cuenta sólo los detectados durante la auditoría). 
  PAR_IN := 'Actualizacion del resultado de la consulta estadistica numero 6.';
  PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_CONSULTA_6';
  
  SELECT anyo INTO vAnyo FROM (SELECT EXTRACT(YEAR FROM i.f_deteccion_incumplimiento) anyo FROM INCUMPLIMIENTO i, AUDITORIA a 
    WHERE i.id_auditoria = a.id_auditoria AND UPPER(a.tipo_auditoria) = 'EXTERNA' AND i.f_deteccion_incumplimiento > a.f_inicio_auditoria 
        GROUP BY EXTRACT(YEAR FROM i.f_deteccion_incumplimiento) ORDER BY EXTRACT(YEAR FROM i.f_deteccion_incumplimiento) DESC) WHERE ROWNUM = 1; 
  
  UPDATE ESTADISTICA SET resultado_consulta = vAnyo WHERE id_consulta = 6;
  RSP := 'RESULTADO ACTUALIZACION CONSULTA 6: '||vAnyo||' - OK.';
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- Si ha saltado una excepcion, la procesamos  
  EXCEPTION
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
END ACTUALIZAR_CONSULTA_6;

PROCEDURE ACTUALIZAR_CONSULTA_7 ( 
   RSP OUT VARCHAR2) AS 
   
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   vTotal_vuln_crit	NUMBER(10);
   vTotal_vuln_crit_no_tot_mit	NUMBER(10);
   vPorcentaje	NUMBER(4,2);
   
  BEGIN
  -- Porcentaje de vulnerabilidades críticas que, en el momento de ejecutar la consulta, tienen alguna acción de mitigación abierta (que no esté en estado “acabada”).
  PAR_IN := 'Actualizacion del resultado de la consulta estadistica numero 7.';
  PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_CONSULTA_7';
  -- NOTA: Previamente habremos actualizado el campo id_estado_vuln de VULNERABILIDAD mediante trigger tras ABM de ACCION.
  -- NOTA 2: Se lanzara tras ABM de VULNERABILIDAD.

  SELECT COUNT(1) INTO vTotal_vuln_crit FROM VULNERABILIDAD WHERE es_vuln_critica = 1; -- Tomamos 1 como VERDADERO, SI es una vulnerabilidad critica.
  SELECT COUNT(1) INTO vTotal_vuln_crit_no_tot_mit FROM VULNERABILIDAD WHERE id_estado_vuln <> 4  AND es_vuln_critica = 1;
  vPorcentaje := ((vTotal_vuln_crit_no_tot_mit/vTotal_vuln_crit)*100);
  
  UPDATE ESTADISTICA SET resultado_consulta = vPorcentaje WHERE id_consulta = 7;
  RSP := 'RESULTADO ACTUALIZACION CONSULTA 7: '||vPorcentaje||' - OK.';
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- Si ha saltado una excepcion, la procesamos  
  EXCEPTION
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		
END ACTUALIZAR_CONSULTA_7; 

PROCEDURE ACTUALIZAR_CONSULTA_8 ( 
   RSP OUT VARCHAR2) AS 
   
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   vAnyo_pasado	NUMBER(4);
   vSesion 	NUMBER(10);
   
  BEGIN
  -- Teniendo en cuenta el último año (el anterior al año en curso), título de la sesión formativa telemática que ha tenido un porcentaje menor de participantes en total.
  PAR_IN := 'Actualizacion del resultado de la consulta estadistica numero 8.';
  PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_CONSULTA_8';
    
  SELECT EXTRACT(YEAR FROM SYSDATE) - 1 INTO vAnyo_pasado FROM DUAL;
  SELECT id_sesion INTO vSesion FROM (SELECT s.id_sesion, COUNT(a.sesion_id_sesion) FROM SESION s, ASIGNADASESION a WHERE EXTRACT(YEAR FROM s.f_inicio_sesion) = vAnyo_pasado AND s.id_sesion = a.sesion_id_sesion GROUP BY s.id_sesion ORDER BY COUNT(a.sesion_id_sesion)) WHERE ROWNUM = 1;
  
  -- Almacenamos el id de sesion en el campo consulta_8 de ESTADISTICA, al visualizar el resultado obtendremos el titulo de la sesion cuyo id hemos almacenado.
  UPDATE ESTADISTICA SET resultado_consulta = vSesion WHERE id_consulta = 8;
  RSP := 'RESULTADO ACTUALIZACION CONSULTA 8: '||vSesion||' - OK.';
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- Si ha saltado una excepcion, la procesamos  
  EXCEPTION
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
END ACTUALIZAR_CONSULTA_8;

PROCEDURE ACTUALIZAR_CONSULTA_9 ( 
   RSP OUT VARCHAR2) AS 
   
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   vNum		NUMBER(4);
   
	BEGIN   
	-- Número de vulnerabilidades críticas detectadas internamente teniendo en cuenta todos los datos de que se dispone. Se consideran detectadas internamente si se detectaron en posterioridad al análisis realizado al inicio del proyecto por la consultora externa.
	PAR_IN := 'Actualizacion del resultado de la consulta estadistica numero 9.';
	PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_CONSULTA_9';
  
	-- Vamos a obtener el numero total de vulnerabilidades CRITICAS detectadas por analisis INTERNO, ya que el primero fue realizado, segun el enunciado, por una empresa externa contratada.
	SELECT COUNT(1) INTO vNum FROM VULNERABILIDAD v, ANALISIS a WHERE v.id_analisis = a.id_analisis AND UPPER(a.tipo_analisis) = 'INTERNO' AND es_vuln_critica = 1 ; 
	
	UPDATE ESTADISTICA SET resultado_consulta = vNum WHERE id_consulta = 9;
	RSP := 'RESULTADO ACTUALIZACION CONSULTA 9: '||vNum||' - OK.';
	INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
	-- Si ha saltado una excepcion, la procesamos  
	EXCEPTION
		WHEN OTHERS THEN 
			RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
			INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
	
END ACTUALIZAR_CONSULTA_9;
  

PROCEDURE ACTUALIZAR_CONSULTA_10 ( 
   RSP OUT VARCHAR2) AS 
   
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   vTotal_acc	NUMBER(10);
   vTotal_acc_en	NUMBER(10);
   vPorcentaje	NUMBER(4,2);   

	BEGIN   
	-- En el momento de ejecutar la consulta, porcentaje de acciones de mitigación en el sistema que están en los estados “en proceso” o “en revisión”.
	PAR_IN := 'Actualizacion del resultado de la consulta estadistica numero 10.';
	PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_CONSULTA_10';
	
	SELECT COUNT(1) INTO vTotal_acc FROM ACCION;
	SELECT COUNT(1) INTO vTotal_acc_en FROM ACCION WHERE id_estado_acc = 2 OR id_estado_acc = 4;
	IF vTotal_acc_en <> 0 THEN 
		vPorcentaje := ((vTotal_acc_en/vTotal_acc)*100);
	ELSE -- El divisor es 0; Ponemos el % a 0 
		vPorcentaje := 0;
	END IF; 
	UPDATE ESTADISTICA SET resultado_consulta = vPorcentaje WHERE id_consulta = 10;
	RSP := 'RESULTADO ACTUALIZACION CONSULTA 10: '||vPorcentaje||' - OK.';
	INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
	-- Si ha saltado una excepcion, la procesamos  
	EXCEPTION
		WHEN OTHERS THEN 
			RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
			INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);

END ACTUALIZAR_CONSULTA_10; 

PROCEDURE ACTUALIZAR_CONSULTA_11 ( 
   RSP OUT VARCHAR2) AS 
   
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   vId_empleado	NUMBER(10);  

	BEGIN   
	-- Teniendo en cuenta todas las acciones de mitigacion en estado “en proceso”, persona responsable con más acciones asignadas.
	PAR_IN := 'Actualizacion del resultado de la consulta estadistica numero 11.';
	PROC := 'PKG_FUNCIONAMIENTO.ACTUALIZAR_CONSULTA_11';
	
	SELECT responsable INTO vId_empleado FROM (SELECT responsable, COUNT(1) acciones FROM ACCION WHERE id_estado_acc = 2 GROUP BY responsable ORDER BY acciones DESC ) WHERE ROWNUM = 1;
	
	-- Almacenamos el id de empleado en el campo consulta_11 de ESTADISTICA, al visualizar el resultado obtendremos el nombre completo del empleado cuyo id hemos almacenado.
	UPDATE ESTADISTICA SET resultado_consulta = vId_empleado WHERE id_consulta = 11;
	RSP := 'RESULTADO ACTUALIZACION CONSULTA 11: '||vId_empleado||' - OK.';
	INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
	  
	-- Si ha saltado una excepcion, la procesamos  
	EXCEPTION
		WHEN OTHERS THEN 
			RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
			INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		
END ACTUALIZAR_CONSULTA_11;

END PKG_FUNCIONAMIENTO;


