CREATE OR REPLACE PACKAGE BODY PKG_ABM AS

/* -- FICHERO PARA CREACION DE CUERPO DE PAQUETE: Cuerpo del paquete para los procedimientos de Altas, Bajas y Modificaciones de los registros de las tablas del sistema
 * Autor: David Giraldez Sanchez
 * Tutor: Jordi Ferrer Duran
 */ 
      
-- NOTA: Acentos eliminados intencionadamente para facilitar la lectura con cualquier configuracion del cliente.

-- ESTE SCRIPT DEBERA LANZARSE COMO USUARIO TFG, O EL USUARIO QUE QUERAMOS UTILIZAR COMO ESQUEMA PRINCIPAL DE LA APLICACION
-- AL LANZAR ESTE SCRIPT COMO USUARIO TFG, EL PAQUETE SE CREA EN SU ESQUEMA Y ES DE SU PROPIEDAD.

-- Se podrian definir procedimientos para modificar uno a uno cada campo de cada tabla, sobrecargando los procedimientos; en general no lo voy a hacer
-- Voy a definir los procedimientos de modificacion de los registros completos
-- Mas adelante se definiran, si son necesarios, aquellos que hagan falta para actualizar un campo de un registro, por operativa del sistema 

/******************
 ***** ACCION *****
 ******************/

PROCEDURE ALTA_ACCION ( 
   pNombre IN ACCION.nombre_accion%TYPE, 
   pF_impl IN ACCION.f_implantacion_acc%TYPE, 
   pF_est IN ACCION.f_estado_acc%TYPE, 
   pURL IN ACCION.url_accion%TYPE, 
   pId_vuln IN ACCION.id_vulnerabilidad%TYPE, 
   pId_est IN ACCION.id_estado_acc%TYPE, 
   pResponsable IN ACCION.responsable%TYPE,
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   compara_fecha	BOOLEAN;
   busca_empleado	NUMBER(1);
   
  BEGIN
  PAR_IN := 'Nombre: '||pNombre||', Fecha Implantacion: '||pF_impl||', Fecha Cambio de Estado: '||pF_est||', URL Documentacion: '||pURL||', Vulnerabilidad: '||pId_vuln||', Estado de la accion: '||pId_est||', Responsable de la accion: '||pResponsable;
  PROC := 'PKG_ABM.ALTA_ACCION';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El nombre de la accion no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha de implantacion de la accion no puede ser nulo.';
  IF pF_impl IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha de cambio de estado de la accion no puede ser nulo.';
  IF pF_est IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El identificador de la vulnerabilidad asociada no puede ser nulo.';
  IF pId_vuln IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El estado de la accion no puede ser nulo.';
  IF pId_est IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El responsable de la accion no puede ser nulo.';
  IF pResponsable IS NULL THEN RAISE MI_EXC; END IF;
    
  -- RESTRICCIONES: Si no hay valores nulos, comprobamos las restricciones de integridad que afecten
  -- El estado de la accion no lo incluimos en estas comprobaciones debido a que hemos considerado que pueden designarse nuevos estados, y habria que modificar el codigo al añadir un nuevo estado; lo comprobara el SGBD al ser clave foranea.
  
  -- La fecha limite de implantacion de una accion de mitigacion siempre sera mayor que la fecha actual.
  RSP := 'RESULTADO: ERROR: La fecha limite de implantacion de la accion debe ser posterior a la fecha actual.';
  compara_fecha := (pF_impl <= SYSDATE); 
  IF compara_fecha THEN RAISE MI_EXC; END IF;
  
  -- La fecha en la que una accion cambia de estado siempre sera anterior o igual a la fecha actual.
  RSP := 'RESULTADO: ERROR: La fecha de cambio de estado de la accion debe ser anterior  o igual a la fecha actual.';
  compara_fecha := (pF_est > SYSDATE); 
  IF compara_fecha THEN RAISE MI_EXC; END IF;
  
  -- El responsable de una accion (al darla de alta) debera ser un empleado en activo.
  -- Comprobamos que el id proporcionado corresponde a un empleado registrado en el sistema, y en activo.
	SELECT COUNT(1) INTO busca_empleado FROM empleado WHERE id_empleado = pResponsable and es_empleado_en_activo = 1;
	RSP := 'RESULTADO: ERROR: El empleado indicado como responsable no existe, o no esta ya en activo en la empresa.';
	IF busca_empleado = 0 THEN RAISE MI_EXC; END IF;
  
  -- INSERCION: Comprobado todo lo anterior, tratamos de insertar el registro.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, o el tipo es incorrecto, saltara excepcion; si no salta, se insertara correctamente.
  INSERT INTO ACCION(nombre_accion, f_implantacion_acc, f_estado_acc, url_accion, id_vulnerabilidad,id_estado_acc,responsable) VALUES (pNombre, pF_impl, pF_est, pURL, pId_vuln, pId_est,pResponsable);
  RSP := 'RESULTADO: OK';
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Tras cada ABM de ACCION, se comprueba si la vulnerabilidad asociada tiene acciones que no esten acabadas, y se actualiza el campo VULNERABILIDAD.id_estado_vuln
  TFG.PKG_FUNCIONAMIENTO.ACTUALIZAR_ESTADO_VULNERABILIDAD(pId_vuln,RSP);
	
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_ACCION;


PROCEDURE BAJA_ACCION (
   pId IN ACCION.id_accion%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC	EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk	NUMBER;
   vId_vuln	NUMBER(10);
   
   BEGIN
    PAR_IN := 'ID: '||pId;
	PROC := 'PKG_ABM.BAJA_ACCION';	
	
    -- Comprobamos que no se esta dejando ningun valor de la PK vacio.
    RSP := 'RESULTADO: ERROR: El codigo de la accion no puede ser nulo.';
    IF pId IS NULL THEN RAISE MI_EXC; END IF;

	-- Comprobamos que el id proporcionado corresponde a una accion registrada en el sistema.
	SELECT COUNT(1) INTO busca_pk FROM accion WHERE id_accion = pId;
	
	IF busca_pk <> 0 THEN 
    -- SI EXISTE, LA ELIMINAMOS; SOLO PUEDE HABER UNA PORQUE ES PK
	-- PERO PRIMERO OBTENEMOS EL ID DE LA VULNERABILIDAD ASOCIADA
		SELECT id_vulnerabilidad INTO vId_vuln FROM ACCION WHERE id_accion = pId;
		RSP := 'RESULTADO: OK';
		DELETE FROM ACCION WHERE id_accion = pId;
		-- Tras cada ABM de ACCION, se comprueba si la vulnerabilidad asociada tiene acciones que no esten acabadas, y se actualiza el campo VULNERABILIDAD.id_estado_vuln
		TFG.PKG_FUNCIONAMIENTO.ACTUALIZAR_ESTADO_VULNERABILIDAD(vId_vuln,RSP);
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: La accion indicada no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_ACCION; 

PROCEDURE MODIFICA_ACCION ( 
   pId_accion IN ACCION.id_accion%TYPE,
   pNombre IN ACCION.nombre_accion%TYPE, 
   pF_impl IN ACCION.f_implantacion_acc%TYPE, 
   pF_est IN ACCION.f_estado_acc%TYPE, 
   pURL IN ACCION.url_accion%TYPE,  
   pId_est IN ACCION.id_estado_acc%TYPE, 
   pResponsable IN ACCION.responsable%TYPE,
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   compara_fecha	BOOLEAN;
   busca_pk	NUMBER;
   consulta_3	NUMBER(6);
   vId_vuln	NUMBER(10);
   
  BEGIN
  PAR_IN := 'Identificador: '||pId_accion||', Nombre: '||pNombre||', Fecha Implementacion: '||pF_impl||', Fecha Estado: '||pF_est||', URL Documentacion: '||pURL||', Estado de la accion: '||pId_est||', Responsable de la accion: '||pResponsable;
  PROC := 'PKG_ABM.MODIFICA_ACCION';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.    
  RSP := 'RESULTADO: ERROR: El codigo de la accion no puede ser nulo.';
  IF pId_accion IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El nombre de la accion no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha de implantacion de la accion no puede ser nulo.';
  IF pF_impl IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha de cambio de estado de la accion no puede ser nulo.';
  IF pF_est IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El estado de la accion no puede ser nulo.';
  IF pId_est IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El responsable de la accion no puede ser nulo.';
  IF pResponsable IS NULL THEN RAISE MI_EXC; END IF;
  
  -- Comprobamos que la accion indicada existe en el sistema
	SELECT COUNT(1) INTO busca_pk FROM accion WHERE id_accion = pId_accion;
	IF busca_pk <> 0 THEN 
    -- SI EXISTE, OBTENEMOS EL ID DE LA VULNERABILIDAD ASOCIADA
		SELECT id_vulnerabilidad INTO vId_vuln FROM ACCION WHERE id_accion = pId_accion;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: La accion indicada no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;		
    
  -- RESTRICCIONES: Si no hay valores nulos, comprobamos las restricciones de integridad que afecten
  -- Comprobamos que el estado exista en la tabla de estados.
  RSP := 'RESULTADO: ERROR: El estado de la accion no existe.';
  SELECT COUNT(1) INTO busca_pk FROM ESTADOACCION WHERE id_estado_acc = pId_est;
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF; -- NO EXISTE 
  
  -- La fecha limite de implantacion de una accion de mitigacion siempre sera mayor que la fecha actual.
  RSP := 'RESULTADO: ERROR: La fecha limite de implantacion de la accion debe ser posterior a la fecha actual.';
  compara_fecha := (pF_impl <= SYSDATE); 
  IF compara_fecha THEN RAISE MI_EXC; END IF;
  
  -- La fecha en la que una accion cambia de estado siempre sera anterior o igual a la fecha actual.
  RSP := 'RESULTADO: ERROR: La fecha de cambio de estado de la accion debe ser anterior  o igual a la fecha actual.';
  compara_fecha := (pF_impl > SYSDATE); 
  IF compara_fecha THEN RAISE MI_EXC; END IF;
  
  -- MODIFICACION: Comprobado todo, tratamos de modificar la accion con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE ACCION SET nombre_accion = pNombre, f_implantacion_acc = pF_impl, f_estado_acc = pF_est, url_accion = pURL, id_estado_acc = pId_est, responsable = presponsable WHERE id_accion = pId_accion;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Tras cada ABM de ACCION, se comprueba si la vulnerabilidad asociada tiene acciones que no esten acabadas, y se actualiza el campo VULNERABILIDAD.id_estado_vuln
  TFG.PKG_FUNCIONAMIENTO.ACTUALIZAR_ESTADO_VULNERABILIDAD(vId_vuln, RSP);
    
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_ACCION;

/********************
 ***** ANALISIS *****
 ********************/
 
PROCEDURE ALTA_ANALISIS ( 
   pNombre IN ANALISIS.nombre_analisis%TYPE, 
   pTipo IN ANALISIS.tipo_analisis%TYPE, 
   pEquipo IN ANALISIS.nombre_equipo%TYPE, 
   pF_inicio IN ANALISIS.f_inicio_analisis%TYPE, 
   pF_fin IN ANALISIS.f_fin_analisis%TYPE, 
   pURL IN ANALISIS.url_analisis%TYPE, 
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   compara_fecha	BOOLEAN;
   
  BEGIN
  PAR_IN := 'Nombre: '||pNombre||', Tipo Analisis: '||pTipo||', Equipo: '||pEquipo||', Fecha Inicio Analisis: '||pF_inicio||', Fecha Fin Analisis: '||pF_fin||', URL de la documentacion: '||pURL;
  PROC := 'PKG_ABM.ALTA_ANALISIS';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El nombre del analisis no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El tipo del analisis no puede ser nulo.';
  IF pTipo IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El nombre del equipo que lleva a cabo el analisis no puede ser nulo.';
  IF pEquipo IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha de inicio del analisis no puede ser nulo.';
  IF pF_inicio IS NULL THEN RAISE MI_EXC; END IF;
    
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- La fecha de inicio de un analisis debe anterior a la fecha de fin, si es que se ha registrado la finalizacion del analisis.
  IF pF_fin IS NOT NULL THEN 
	RSP := 'RESULTADO: ERROR: La fecha de inicio de un analisis debe ser anterior a la fecha de fin.';
	compara_fecha := (pF_inicio > pF_fin); 
	IF compara_fecha THEN RAISE MI_EXC; END IF;
  END IF;
  
  -- INSERCION: Comprobado todo, tratamos de modificar el registro.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  INSERT INTO ANALISIS(nombre_analisis, tipo_analisis, nombre_equipo, f_inicio_analisis, f_fin_analisis, url_analisis) VALUES (pNombre, pTipo, pEquipo, pF_inicio, pF_fin, pURL);
  RSP := 'RESULTADO: OK';
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- En este caso, ninguno.
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_ANALISIS;


PROCEDURE BAJA_ANALISIS (
   pId IN ANALISIS.id_analisis%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk	NUMBER;
   
   BEGIN
    PAR_IN := 'ID: '||pId;
	PROC := 'PKG_ABM.BAJA_ANALISIS';	
	
    -- Comprobamos que no se esta dejando ningun valor de la PK vacio.
    RSP := 'RESULTADO: ERROR: El codigo del analisis no puede ser nulo.';
    IF pId IS NULL THEN RAISE MI_EXC; END IF;

	-- Comprobamos que el id proporcionado corresponde a un analisis registrado en el sistema.
	SELECT COUNT(1) INTO busca_pk FROM analisis WHERE id_analisis = pId;
	
	IF busca_pk <> 0 THEN 
    -- SI EXISTE, LA ELIMINAMOS; SOLO PUEDE HABER UNA PORQUE ES PK
		RSP := 'RESULTADO: OK';
		DELETE FROM ANALISIS WHERE id_analisis = pId;
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: El analisis indicado no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_ANALISIS;


PROCEDURE MODIFICA_ANALISIS ( 
   pId IN ANALISIS.id_analisis%TYPE, 
   pNombre IN ANALISIS.nombre_analisis%TYPE, 
   pTipo IN ANALISIS.tipo_analisis%TYPE, 
   pEquipo IN ANALISIS.nombre_equipo%TYPE, 
   pF_inicio IN ANALISIS.f_inicio_analisis%TYPE, 
   pF_fin IN ANALISIS.f_fin_analisis%TYPE, 
   pURL IN ANALISIS.url_analisis%TYPE, 
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   compara_fecha	BOOLEAN;
   
  BEGIN
  PAR_IN := 'Identificador: '||pId||', Nombre: '||pNombre||', Tipo Analisis: '||pTipo||', Equipo: '||pEquipo||', Fecha Inicio Analisis: '||pF_inicio||', Fecha Fin Analisis: '||pF_fin||', URL de la documentacion: '||pURL;
  PROC := 'PKG_ABM.MODIFICA_ANALISIS';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El codigo del analisis no puede ser nulo.';
  IF pId IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El nombre del analisis no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El tipo del analisis no puede ser nulo.';
  IF pTipo IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El nombre del equipo que lleva a cabo el analisis no puede ser nulo.';
  IF pEquipo IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha de inicio del analisis no puede ser nulo.';
  IF pF_inicio IS NULL THEN RAISE MI_EXC; END IF;
    
  -- RESTRICCIONES: Si no hay valores nulos, comprobamos las restricciones de integridad que afecten
  -- El tipo del analisis sera INTERNO o EXTERNO, no hay mas opciones.
  RSP := 'RESULTADO: ERROR: El tipo del analisis debe ser INTERNO o EXTERNO.';
  IF (UPPER(pTipo) <> 'EXTERNO' AND UPPER(pTipo) <> 'INTERNO') THEN RAISE MI_EXC; END IF;
  
  -- La fecha de inicio de un analisis debe anterior a la fecha de fin, si es que se ha registrado la finalizacion del analisis.
  IF pF_fin IS NOT NULL THEN 
	RSP := 'RESULTADO: ERROR: La fecha de inicio de un analisis debe ser anterior a la fecha de fin.';
	compara_fecha := (pF_inicio > pF_fin); 
	IF compara_fecha THEN RAISE MI_EXC; END IF;
  END IF;
 
  -- MODIFICACION: Comprobado todo, tratamos de modificar el analisis con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE ANALISIS SET nombre_analisis = pNombre, tipo_analisis = UPPER(pTipo), nombre_equipo = pEquipo, f_inicio_analisis = pF_inicio, f_fin_analisis = pF_fin, url_analisis = pURL WHERE id_analisis = pId;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
 
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- En este caso, ninguno.
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_ANALISIS;

/**************************
 ***** ASIGNADASESION *****
 ***************************/

PROCEDURE ALTA_ASIGNADASESION ( 
   pSesion IN ASIGNADASESION.sesion_id_sesion%TYPE, 
   pEmpleado IN ASIGNADASESION.empleado_id_empleado%TYPE, 
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   compara_fecha	BOOLEAN;
   fecha_sesion		DATE;
   vDepto NUMBER(10);   
   busca_pk	NUMBER;
   
  BEGIN
  PAR_IN := 'Sesion: '||pSesion||', Empleado: '||pEmpleado;
  PROC := 'PKG_ABM.ALTA_ASIGNADASESION';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de sesion no puede ser nulo.';
  IF pSesion IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El identificador de empleado no puede ser nulo.';
  IF pEmpleado IS NULL THEN RAISE MI_EXC; END IF;
  -- Asumiremos que como estamos dando de alta la asignacion, aun no se ha finalizado la sesion por parte del empleado, y lo registraremos asi.
  
  	-- Comprobamos que los id proporcionados identifican a una sesion y a un empleado registrados en el sistema.
	SELECT COUNT(1) INTO busca_pk FROM SESION WHERE id_sesion = pSesion;
	RSP := 'RESULTADO: ERROR: La sesion indicada no existe.';
	IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
	SELECT COUNT(1) INTO busca_pk FROM EMPLEADO  WHERE id_empleado = pEmpleado;
	IF busca_pk <> 0 THEN 
    -- SI EXISTE, OBTENEMOS EL DEPARTAMENTO DEL EMPLEADO
		SELECT id_departamento INTO vDepto FROM EMPLEADO WHERE id_empleado = pEmpleado;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: El empleado indicado no existe.';
		RAISE MI_EXC;
	END IF;
   
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- Ninguna en este caso.
  
  -- INSERCION: Comprobado todo, tratamos de insertar el registro.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  INSERT INTO ASIGNADASESION(sesion_id_sesion , empleado_id_empleado, ha_finalizado_sesion) VALUES (pSesion, pEmpleado, 0);
  RSP := 'RESULTADO: OK';
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Con cada alta, baja o modificacion de asignacion de sesion, se actualiza el campo correspondiente en DEPARTAMENTO, y en este caso NO  se anota en la tabla estadística
  TFG.PKG_FUNCIONAMIENTO.ACTUALIZAR_PORCENTAJE_DE_DEPARTAMENTO(vDepto,RSP);
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_ASIGNADASESION;


PROCEDURE BAJA_ASIGNADASESION (
   pSesion IN ASIGNADASESION.sesion_id_sesion%TYPE, 
   pEmpleado IN ASIGNADASESION.empleado_id_empleado%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk	NUMBER;
   vDepto NUMBER(10);
   
   BEGIN
    PAR_IN := 'Sesion: '||pSesion||', Empleado: '||pEmpleado;
	PROC := 'PKG_ABM.BAJA_ASIGNADASESION';	
	
    -- Comprobamos que no se esta dejando ningun valor de la PK vacio.
    RSP := 'RESULTADO: ERROR: El codigo de la sesion no puede ser nulo.';
    IF pSesion IS NULL THEN RAISE MI_EXC; END IF;
	RSP := 'RESULTADO: ERROR: El codigo del empleado no puede ser nulo.';
    IF pEmpleado IS NULL THEN RAISE MI_EXC; END IF;

	-- Comprobamos que los id proporcionados identifican a una asignacion de sesion registrada en el sistema.
	SELECT COUNT(1) INTO busca_pk FROM ASIGNADASESION WHERE sesion_id_sesion = pSesion AND empleado_id_empleado = pEmpleado;
	
	IF busca_pk <> 0 THEN 
    -- SI EXISTE, LA ELIMINAMOS; SOLO PUEDE HABER UNA PORQUE ES PK
	-- Primero obtenemos el departamento del empleado
		SELECT id_departamento INTO vDepto FROM EMPLEADO WHERE id_empleado = pEmpleado;
		RSP := 'RESULTADO: OK';
		DELETE FROM ASIGNADASESION WHERE sesion_id_sesion = pSesion AND empleado_id_empleado = pEmpleado;
	-- Con cada alta, baja o modificacion de asignacion de sesion, se actualiza el campo correspondiente en DEPARTAMENTO
		TFG.PKG_FUNCIONAMIENTO.ACTUALIZAR_PORCENTAJE_DE_DEPARTAMENTO(vDepto, RSP);
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: La asignacion de sesion indicada no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_ASIGNADASESION;


PROCEDURE MODIFICA_ASIGNADASESION ( 
   pSesion IN ASIGNADASESION.sesion_id_sesion%TYPE, 
   pEmpleado IN ASIGNADASESION.empleado_id_empleado%TYPE, 
   pFinalizada IN ASIGNADASESION.ha_finalizado_sesion%TYPE,
   pF_fin IN ASIGNADASESION.f_fin_sesion%TYPE,
   RSP OUT VARCHAR2) AS
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
   fecha_sesion		DATE;
   vFinalizada		NUMBER(1);
   busca_pk	NUMBER;
   vDepto NUMBER(10);
   
  BEGIN
  PAR_IN := 'Sesion: '||pSesion||', Empleado: '||pEmpleado||', Sesion Finalizada: '||pFinalizada||', Fecha Finalizacion de Sesion: '||pF_fin;
  PROC := 'PKG_ABM.MODIFICA_ASIGNADASESION';
  vFinalizada := pFinalizada;
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de sesion no puede ser nulo.';
  IF pSesion IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El identificador de empleado no puede ser nulo.';
  IF pEmpleado IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El indicador de finalizacion de sesion no puede ser nulo.';
  IF pFinalizada IS NULL THEN RAISE MI_EXC; END IF;
  
  -- Comprobamos que los id proporcionados identifican a una asignacion de sesion registrada en el sistema.
	SELECT COUNT(1) INTO busca_pk FROM ASIGNADASESION WHERE sesion_id_sesion = pSesion AND empleado_id_empleado = pEmpleado;
	
	IF busca_pk <> 0 THEN 
    -- SI EXISTE OBTENEMOS EL DEPARTAMENTO DEL EMPLEADO
		SELECT id_departamento INTO vDepto FROM EMPLEADO WHERE id_empleado = pEmpleado;
		RSP := 'RESULTADO: OK';
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: La asignacion de sesion indicada no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;
  
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  
  -- La fecha de fin de una sesion asignada debe posterior a la fecha de inicio de la sesion.
  IF pF_fin IS NOT NULL THEN
	SELECT f_inicio_sesion INTO fecha_sesion FROM SESION WHERE id_sesion = pSesion;
	RSP := 'RESULTADO: ERROR: La fecha de finalizacion de la sesion asignada debe debe ser posterior a la fecha de inicio de la sesion.';
	IF (pF_fin < fecha_sesion) THEN RAISE MI_EXC; 
	ELSE
		-- Si se ha proporcionado una fecha de fin de la sesion asignada, se ignorara el valor proporcionado para el indicador de finalizacion de sesion, y se pondra a 1 (Finalizada).
		vFinalizada := 1;
	END IF;
  END IF;
  
  -- En la definicion de la tabla se ha añadido una restriccion que comprueba los valores correctos para el campo ha_finalizado_sesion, que deben ser 0 o 1.
  -- Los comprobamos antes aqui para personalizar el mensaje en el LOGOPERACIONES.
  RSP := 'RESULTADO: ERROR: El indicador de finalizacion de sesion debe ser 0 o 1.';
  IF (vFinalizada <> 0 AND vFinalizada <> 1) THEN RAISE MI_EXC; END IF;
  
   -- MODIFICACION: Comprobado todo, tratamos de modificar el analisis con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE ASIGNADASESION SET ha_finalizado_sesion = vFinalizada, f_fin_sesion = pF_fin WHERE sesion_id_sesion = pSesion AND empleado_id_empleado = pEmpleado;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Con cada alta, baja o modificacion de asignacion de sesion, se actualiza el campo correspondiente en DEPARTAMENTO, y en este caso NO  se anota en la tabla estadística
  TFG.PKG_FUNCIONAMIENTO.ACTUALIZAR_PORCENTAJE_DE_DEPARTAMENTO(vDepto,RSP);
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_ASIGNADASESION;

/*********************
 ***** AUDITORIA *****
 *********************/

PROCEDURE ALTA_AUDITORIA ( 
   pNombre IN AUDITORIA.nombre_auditoria%TYPE, 
   pTipo IN AUDITORIA.tipo_auditoria%TYPE,
   pF_inicio IN AUDITORIA.f_inicio_auditoria%TYPE,
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   compara_fecha BOOLEAN;
   
  BEGIN
  PAR_IN := 'Nombre Auditoria: '||pNombre||', Tipo Auditoria: '||pTipo||', Fecha Inicio Auditoria: '||pF_inicio;
  PROC := 'PKG_ABM.ALTA_AUDITORIA';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El nombre de la auditoria no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El tipo de la auditoria no puede ser nulo.';
  IF pTipo IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha de inicio de la auditoria no puede ser nulo.';
  IF pF_inicio IS NULL THEN RAISE MI_EXC; END IF;
    
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- Ninguna en este caso.
  
  -- INSERCION: Comprobado todo, tratamos de modificar el registro.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  INSERT INTO AUDITORIA(nombre_auditoria, tipo_auditoria, f_inicio_auditoria) VALUES (pNombre, pTipo, pF_inicio);
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso. 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_AUDITORIA;


PROCEDURE BAJA_AUDITORIA (
   pId IN AUDITORIA.id_auditoria%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk	NUMBER;
   
   BEGIN
    PAR_IN := 'Id Auditoria: '||pId;
	PROC := 'PKG_ABM.BAJA_AUDITORIA';	
	
    -- Comprobamos que no se esta dejando ningun valor de la PK vacio.
    RSP := 'RESULTADO: ERROR: El identificador de la auditoria no puede ser nulo.';
    IF pId IS NULL THEN RAISE MI_EXC; END IF;

	-- Comprobamos que el id proporcionado corresponde a una auditoria registrada en el sistema.
	SELECT COUNT(1) INTO busca_pk FROM AUDITORIA WHERE id_auditoria = pId;
	
	IF busca_pk <> 0 THEN 
    -- SI EXISTE, LA ELIMINAMOS; SOLO PUEDE HABER UNA PORQUE ES PK
		RSP := 'RESULTADO: OK';
		DELETE FROM AUDITORIA WHERE id_auditoria = pId;
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: La asignacion de sesion no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_AUDITORIA;


PROCEDURE MODIFICA_AUDITORIA ( 
   pId IN AUDITORIA.id_auditoria%TYPE, 
   pNombre IN AUDITORIA.nombre_auditoria%TYPE, 
   pTipo IN AUDITORIA.tipo_auditoria%TYPE,
   pF_inicio IN AUDITORIA.f_inicio_auditoria%TYPE,
   pF_fin IN AUDITORIA.f_fin_auditoria%TYPE,
   RSP OUT VARCHAR2) AS
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
  BEGIN
  PAR_IN := 'Identificador Auditoria: '||pId||', Nombre Auditoria: '||pNombre||', Tipo Auditoria: '||pTipo||', Fecha Inicio Auditoria: '||pF_inicio||', Fecha Fin Auditoria: '||pF_fin;
  PROC := 'PKG_ABM.MODIFICA_AUDITORIA';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El nombre de la auditoria no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El tipo de la auditoria no puede ser nulo.';
  IF pTipo IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha de inicio de la auditoria no puede ser nulo.';
  IF pF_inicio IS NULL THEN RAISE MI_EXC; END IF;
    
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- La fecha de inicio de una auditoria siempre será menor que su fecha de fin, si es que esta está registrada.
  IF pF_fin IS NOT NULL THEN 
	RSP := 'RESULTADO: ERROR: La fecha de inicio de una auditoria debe ser anterior a la fecha de fin.';
	IF (pF_inicio > pF_fin) THEN RAISE MI_EXC; END IF;
  END IF;
  
   -- MODIFICACION: Comprobado todo, tratamos de modificar el analisis con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE AUDITORIA SET nombre_auditoria = pNombre, tipo_auditoria = pTipo, f_inicio_auditoria = pF_inicio , f_fin_auditoria = pF_fin WHERE id_auditoria = pId;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- TODO 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_AUDITORIA;
  
/************************
 ***** DEPARTAMENTO *****
 ************************/

PROCEDURE ALTA_DEPARTAMENTO ( 
   pNombre IN DEPARTAMENTO.nombre_depto%TYPE, 
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
  BEGIN
  PAR_IN := 'Nombre: '||pNombre;
  PROC := 'PKG_ABM.ALTA_DEPARTAMENTO';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El nombre del departamento no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
    
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- Ninguna en este caso
  
  -- INSERCION: Comprobado todo, tratamos de modificar el registro.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  INSERT INTO DEPARTAMENTO(nombre_depto) VALUES (pNombre);
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso. 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_DEPARTAMENTO;


PROCEDURE BAJA_DEPARTAMENTO (
   pId IN DEPARTAMENTO.id_departamento%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
   busca_pk	NUMBER(1);
   
   BEGIN
    PAR_IN := 'Id Departamento: '||pId;
	PROC := 'PKG_ABM.BAJA_DEPARTAMENTO';	
	
    -- Comprobamos que no se esta dejando ningun valor de la PK vacio.
    RSP := 'RESULTADO: ERROR: El identificador del departamento no puede ser nulo.';
    IF pId IS NULL THEN RAISE MI_EXC; END IF;

	-- Comprobamos que el id proporcionado corresponde a un departamento registrado en el sistema.
	SELECT COUNT(1) INTO busca_pk FROM DEPARTAMENTO WHERE id_departamento = pId;
	
	IF busca_pk <> 0 THEN 
    -- SI EXISTE, LA ELIMINAMOS; SOLO PUEDE HABER UNA PORQUE ES PK
		RSP := 'RESULTADO: OK';
		DELETE FROM DEPARTAMENTO WHERE id_departamento = pId;
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: El departamento no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_DEPARTAMENTO;


PROCEDURE MODIFICA_DEPARTAMENTO ( 
   pId IN DEPARTAMENTO.id_departamento%TYPE, 
   pNombre IN DEPARTAMENTO.nombre_depto%TYPE, 
   RSP OUT VARCHAR2) AS
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
   busca_pk	NUMBER(1);
   
  BEGIN
  -- NOTA: Los campos incumplimientos_empleados y empls_con_sesiones_pendientes de la tabla de departamentos, si bien son campos que no admiten valores nulos, se inicializan a 0 al insertar el departamento, y se actualizan de forma automática con la operativa del sistema. Estos campos no se modificarán mediante este procedimiento.
  PAR_IN := 'Identificador: '||pId||', Nombre: '||pNombre;
  PROC := 'PKG_ABM.MODIFICA_DEPARTAMENTO';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de departamento no puede ser nulo.';
  IF pId IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El nombre del departamento no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
  
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- Ninguna en este caso
  
   -- MODIFICACION: Comprobado todo, tratamos de modificar el departamento con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE DEPARTAMENTO SET nombre_depto = pNombre WHERE id_departamento = pId;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- TODO 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_DEPARTAMENTO;

  
/********************
 ***** EMPLEADO *****
 ********************/

PROCEDURE ALTA_EMPLEADO (  
   pDNI IN EMPLEADO.dni_empleado%TYPE,
   pNombre IN EMPLEADO.nombre_emp%TYPE,  
   pAp1 IN EMPLEADO.apellido1_emp%TYPE, 
   pAp2 IN EMPLEADO.apellido2_emp%TYPE, 
   pEmail IN EMPLEADO.email_empleado%TYPE, 
   pTfno IN EMPLEADO.tfno_empleado%TYPE,  
   pDepto IN EMPLEADO.id_departamento%TYPE,
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk	NUMBER(1);
   
  BEGIN
  PAR_IN := 'DNI: '||pDNI||', Nombre: '||pNombre||', Apellido 1: '||pAp1||', Apellido 2: '||pAp2||', Email: '||pEmail||', Telefono de contacto: '||pTfno||', Departamento: '||pDepto;
  PROC := 'PKG_ABM.ALTA_EMPLEADO';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El DNI del empleado no puede ser nulo.';
  IF pDNI IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El nombre del empleado no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El primer apellido del empleado no puede ser nulo.';
  IF pAp1 IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El segundo apellido del empleado no puede ser nulo.';
  IF pAp2 IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El email del empleado no puede ser nulo.';
  IF pEmail IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El telefono de contacto del empleado no puede ser nulo.';
  IF pTfno IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El departamento del empleado no puede ser nulo.';
  IF pDepto IS NULL THEN RAISE MI_EXC; END IF;
    
  -- NOTA: Los campos incumplimientos_emp y es_empleado_en_activo de la tabla de empleados, si bien son campos que no admiten valores nulos, se inicializan a 0 y 1 respectivamente, y se actualizan de forma automática con la operativa del sistema. Asumimos que, si se está dando de alta a un empleado, es porque está en activo, y aun no ha cometido incumplimientos de politicas.
	
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- Aunque el sistema comprobará que el departamento exista, ya que es clave foránea, vamos a comprobarlo previamente a la inserción, para personalizar el mensaje en el log de operaciones en caso necesario.
  
  -- Comprobamos que el id proporcionado corresponde a un departamento registrado en el sistema.
  SELECT COUNT(1) INTO busca_pk FROM DEPARTAMENTO WHERE id_departamento = pDepto;
  -- SI NO EXISTE
  RSP := 'RESULTADO: ERROR: El departamento indicado no existe.';
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  -- SI EXISTE, INSERTAMOS EL REGISTRO DE EMPLEADO
  
  -- INSERCION: Comprobado todo, tratamos de insertar el registro.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  INSERT INTO EMPLEADO(dni_empleado, nombre_emp, apellido1_emp, apellido2_emp, email_empleado, tfno_empleado, es_empleado_en_activo, incumplimientos_emp, id_departamento) VALUES (pDNI, pNombre, pAp1, pAp2, pEmail, pTfno, 1, 0, pDepto);
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- TODO  
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_EMPLEADO;


PROCEDURE BAJA_EMPLEADO (
   pId IN EMPLEADO.id_empleado%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk	NUMBER;
   
   BEGIN
    PAR_IN := 'Id Empleado: '||pId;
	PROC := 'PKG_ABM.BAJA_EMPLEADO';	
	
    -- Comprobamos que no se esta dejando ningun valor de la PK vacio.
    RSP := 'RESULTADO: ERROR: El identificador del empleado no puede ser nulo.';
    IF pId IS NULL THEN RAISE MI_EXC; END IF;

	-- Comprobamos que el id proporcionado corresponde a un empleado registrado en el sistema.
	SELECT COUNT(1) INTO busca_pk FROM EMPLEADO WHERE id_empleado = pId;
	
	IF busca_pk <> 0 THEN 
    -- SI EXISTE, LO ELIMINAMOS; SOLO PUEDE HABER UNO PORQUE ES PK
		RSP := 'RESULTADO: OK';
		DELETE FROM EMPLEADO WHERE id_empleado = pId;
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: El empleado no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_EMPLEADO;


PROCEDURE MODIFICA_EMPLEADO ( 
   pId IN EMPLEADO.id_empleado%TYPE, 
   pDNI IN EMPLEADO.dni_empleado%TYPE,
   pNombre IN EMPLEADO.nombre_emp%TYPE,  
   pAp1 IN EMPLEADO.apellido1_emp%TYPE, 
   pAp2 IN EMPLEADO.apellido2_emp%TYPE, 
   pEmail IN EMPLEADO.email_empleado%TYPE, 
   pTfno IN EMPLEADO.tfno_empleado%TYPE,  
   pActivo IN EMPLEADO.es_empleado_en_activo%TYPE, 
   pDepto IN EMPLEADO.id_departamento%TYPE,
   RSP OUT VARCHAR2) AS
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
   busca_pk	NUMBER(1);
   
  BEGIN
  PAR_IN := 'ID Empleado: '||pId||', DNI: '||pDNI||', Nombre: '||pNombre||', Apellido 1: '||pAp1||', Apellido 2: '||pAp2||', Email: '||pEmail||', Telefono de contacto: '||pTfno||', En Activo: '||pActivo||', Departamento: '||pDepto;
  PROC := 'PKG_ABM.MODIFICA_EMPLEADO';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de empleado no puede ser nulo.';
  IF pId IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El DNI del empleado no puede ser nulo.';
  IF pDNI IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El nombre del empleado no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El primer apellido del empleado no puede ser nulo.';
  IF pAp1 IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El segundo apellido del empleado no puede ser nulo.';
  IF pAp2 IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El email del empleado no puede ser nulo.';
  IF pEmail IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El telefono de contacto del empleado no puede ser nulo.';
  IF pTfno IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El departamento del empleado no puede ser nulo.';
  IF pDepto IS NULL THEN RAISE MI_EXC; END IF;
  
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
 -- Aunque el sistema comprobará que el departamento exista, ya que es clave foránea, vamos a comprobarlo previamente a la inserción, para personalizar el mensaje en el log de operaciones.
  
  -- Comprobamos que el id proporcionado corresponde a un departamento registrado en el sistema.
  SELECT COUNT(1) INTO busca_pk FROM DEPARTAMENTO WHERE id_departamento = pDepto;
  -- SI NO EXISTE
  RSP := 'RESULTADO: ERROR: El departamento indicado no existe.';
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  -- SI EXISTE, INSERTAMOS EL REGISTRO DE EMPLEADO
  
   -- MODIFICACION: Comprobado todo, tratamos de modificar el analisis con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE EMPLEADO SET dni_empleado = pDNI , nombre_emp = pNombre, apellido1_emp = pAp1 , apellido2_emp = pAp2, email_empleado = pEmail , tfno_empleado = pTfno , es_empleado_en_activo = pActivo, id_departamento = pDepto WHERE id_empleado = pId;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- TODO 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_EMPLEADO;


/***********************
 ***** ESTADISTICA *****
 ***********************/

-- Las actualizaciones de esta tabla se realizarán desde procedimientos del paquete PKG_ESTADISTICAS

/************************
 ***** ESTADOACCION *****
 ************************/
 
PROCEDURE ALTA_ESTADOACCION ( 
   pId_estado IN ESTADOACCION.id_estado_acc%TYPE,
   pEstado IN ESTADOACCION.estado_accion%TYPE, 
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
   busca_pk NUMBER(1);
   
  BEGIN
  PAR_IN := 'Identificador: '||pId_estado||', Estado: '||pEstado;
  PROC := 'PKG_ABM.ALTA_ESTADOACCION';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de estado de accion no puede ser nulo.';
  IF pId_estado IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El estado de accion no puede ser nulo.';
  IF pEstado IS NULL THEN RAISE MI_EXC; END IF;
    
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- Comprobamos que la clave primaria no exista ya en el sistema, para presentar en el log un mensaje personalizado en caso de ser necesario.
  SELECT COUNT(1) INTO busca_pk FROM ESTADOACCION WHERE id_estado_acc = pId_estado;
  -- SI YA EXISTE
  RSP := 'RESULTADO: ERROR: El identificador indicado ya existe.';
  IF busca_pk <> 0 THEN RAISE MI_EXC; END IF;
  
  -- INSERCION: Comprobado todo, tratamos de modificar el registro.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  INSERT INTO ESTADOACCION(id_estado_acc, estado_accion) VALUES (pId_estado,pEstado);

  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso. 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_ESTADOACCION;


PROCEDURE BAJA_ESTADOACCION (
   pId IN ESTADOACCION.id_estado_acc%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk	NUMBER(1);
   
   BEGIN
    PAR_IN := 'Identificador: '||pId;
	PROC := 'PKG_ABM.BAJA_ESTADOACCION';	
	
    -- Comprobamos que no se esta dejando ningun valor de la PK vacio.
    RSP := 'RESULTADO: ERROR: El identificador del estado de accion no puede ser nulo.';
    IF pId IS NULL THEN RAISE MI_EXC; END IF;

	-- Comprobamos que el id proporcionado corresponde a un estado de accion registrado en el sistema.
	SELECT COUNT(1) INTO busca_pk FROM ESTADOACCION WHERE id_estado_acc = pId;
	
	IF busca_pk <> 0 THEN 
    -- SI EXISTE, LO ELIMINAMOS; SOLO PUEDE HABER UNO PORQUE ES PK
		RSP := 'RESULTADO: OK';
		DELETE FROM ESTADOACCION WHERE id_estado_acc = pId;
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: El estado indicado no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_ESTADOACCION;


PROCEDURE MODIFICA_ESTADOACCION ( 
   pId IN ESTADOACCION.id_estado_acc%TYPE, 
   pEstado IN ESTADOACCION.estado_accion%TYPE, 
   RSP OUT VARCHAR2) AS
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
   busca_pk NUMBER(1);
   
  BEGIN
  PAR_IN := 'Identificador: '||pId||', Estado: '||pEstado;
  PROC := 'PKG_ABM.MODIFICA_ESTADOACCION';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de estado no puede ser nulo.';
  IF pId IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El estado no puede ser nulo.';
  IF pEstado IS NULL THEN RAISE MI_EXC; END IF;
    
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- Comprobamos que el estado que queremos modificar esta registrado en el sistema.
  SELECT COUNT(1) INTO busca_pk FROM ESTADOACCION WHERE id_estado_acc = pId;
  -- SI NO EXISTE, NO PODEMOS MODIFICARLO 
  RSP := 'RESULTADO: ERROR: El identificador indicado no existe.';
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
 
   -- MODIFICACION: Comprobado todo, tratamos de modificar el analisis con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE ESTADOACCION SET estado_accion = pEstado WHERE id_estado_acc = pId;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_ESTADOACCION;
 
/**********************
 ***** ESTADOVULN *****
 **********************/
 
PROCEDURE ALTA_ESTADOVULN ( 
   pId_estado IN ESTADOVULN.id_estado_vuln%TYPE,
   pEstado IN ESTADOVULN.estado_vuln%TYPE, 
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk NUMBER(1);
   
  BEGIN
  PAR_IN := 'Identificador Vuln: '||pId_estado||', Estado: '||pEstado;
  PROC := 'PKG_ABM.ALTA_ESTADOVULN';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de estado no puede ser nulo.';
  IF pId_estado IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El estado no puede ser nulo.';
  IF pEstado IS NULL THEN RAISE MI_EXC; END IF;
    
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- Comprobamos que la clave primaria no exista ya en el sistema, para presentar en el log un mensaje personalizado en caso de ser necesario.
  SELECT COUNT(1) INTO busca_pk FROM ESTADOVULN WHERE id_estado_vuln = pId_estado;
  -- SI YA EXISTE
  RSP := 'RESULTADO: ERROR: El identificador indicado ya existe.';
  IF busca_pk <> 0 THEN RAISE MI_EXC; END IF;
  
  -- INSERCION: Comprobado todo, tratamos de modificar el registro.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  INSERT INTO ESTADOVULN(id_estado_vuln,estado_vuln) VALUES (pId_estado,pEstado);
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso. 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_ESTADOVULN;


PROCEDURE BAJA_ESTADOVULN (
   pId IN ESTADOVULN.id_estado_vuln%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk	NUMBER(1);
   
   BEGIN
    PAR_IN := 'Identificador: '||pId;
	PROC := 'PKG_ABM.BAJA_ESTADOVULN';	
	
    -- Comprobamos que no se esta dejando ningun valor de la PK vacio.
    RSP := 'RESULTADO: ERROR: El identificador del estado no puede ser nulo.';
    IF pId IS NULL THEN RAISE MI_EXC; END IF;

	-- Comprobamos que el id proporcionado corresponde a un estado registrado en el sistema.
	SELECT COUNT(1) INTO busca_pk FROM ESTADOVULN WHERE id_estado_vuln = pId;
	
	IF busca_pk <> 0 THEN 
    -- SI EXISTE, LO ELIMINAMOS; SOLO PUEDE HABER UNO PORQUE ES PK
		RSP := 'RESULTADO: OK';
		DELETE FROM ESTADOVULN WHERE id_estado_vuln = pId;
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- NO EXISTE
		RSP := 'RESULTADO: ERROR: El estado no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_ESTADOVULN;


PROCEDURE MODIFICA_ESTADOVULN ( 
   pId IN ESTADOVULN.id_estado_vuln%TYPE, 
   pEstado IN ESTADOVULN.estado_vuln%TYPE, 
   RSP OUT VARCHAR2) AS
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk NUMBER(1);
   
  BEGIN
  PAR_IN := 'Identificador: '||pId||', Estado: '||pEstado;
  PROC := 'PKG_ABM.MODIFICA_ESTADOVULN';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de estado no puede ser nulo.';
  IF pId IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El estado no puede ser nulo.';
  IF pEstado IS NULL THEN RAISE MI_EXC; END IF;
    
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- Comprobamos que el estado que queremos modificar esta registrado en el sistema.
  SELECT COUNT(1) INTO busca_pk FROM ESTADOVULN WHERE id_estado_vuln = pId;
  -- SI NO EXISTE, NO PODEMOS MODIFICARLO 
  RSP := 'RESULTADO: ERROR: El identificador indicado no existe.';
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  
   -- MODIFICACION: Comprobado todo, tratamos de modificar el analisis con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE ESTADOVULN SET estado_vuln = pEstado WHERE id_estado_vuln = pId;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_ESTADOVULN;
 
/**************************
 ***** ESTUDIOPROCESO *****
 **************************/
 
PROCEDURE ALTA_ESTUDIOPROCESO ( 
   pAnalisis IN ESTUDIOPROCESO.id_analisis%TYPE, 
   pProceso IN ESTUDIOPROCESO.id_proceso%TYPE, 
   pFecha IN ESTUDIOPROCESO.f_estudioproceso%TYPE, 
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
  
   busca_pk NUMBER(1);
   busca_pk2 NUMBER(1);
   fecha_inicio_analisis	DATE;
   fecha_fin_analisis	DATE;
   
  BEGIN
  PAR_IN := 'Analisis: '||pAnalisis||', Proceso: '||pProceso||', Fecha del estudio: '||pFecha;
  PROC := 'PKG_ABM.ALTA_ESTUDIOPROCESO';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El analisis no puede ser nulo.';
  IF pAnalisis IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El proceso no puede ser nulo.';
  IF pProceso IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha del estudio no puede ser nula.';
  IF pFecha IS NULL THEN RAISE MI_EXC; END IF;
  
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- La fecha de estudio de un proceso durante un analisis de vulnerabilidades siempre sera mayor o igual a la fecha de inicio del analisis, y menor o igual a la fecha de fin del analisis (si esta registrada).
  -- Comprobamos que el id proporcionado corresponde a un analisis registrado en el sistema.
  SELECT COUNT(1) INTO busca_pk FROM ANALISIS WHERE id_analisis = pAnalisis;
  IF busca_pk <> 0 THEN 
	-- SI EXISTE, OBTENEMOS LAS FECHAS QUE NOS INTERESAN
	SELECT f_inicio_analisis INTO fecha_inicio_analisis FROM ANALISIS WHERE id_analisis = pAnalisis;
	SELECT f_fin_analisis INTO fecha_fin_analisis FROM ANALISIS WHERE id_analisis = pAnalisis;
  ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: El analisis no existe.';
		RAISE MI_EXC;
	END IF;
	
  -- Comprobamos las fechas	
  IF pFecha > fecha_inicio_analisis THEN 
	IF fecha_fin_analisis IS NOT NULL THEN 
		IF pFecha > fecha_fin_analisis THEN 
		-- La fecha de estudio no coincide con la fecha de fin del analisis
			RSP := 'RESULTADO: ERROR: La fecha de estudio es posterior a la fecha de fin del analisis.';
			RAISE MI_EXC;
		END IF;
	END IF;
  ELSE 
  -- La fecha de estudio no coincide con la fecha de inicio del analisis
	RSP := 'RESULTADO: ERROR: La fecha de estudio es anterior a la fecha de inicio del analisis.';
	RAISE MI_EXC;
  END IF;
  
  -- Comprobamos que el id proporcionado corresponde a un proceso registrado en el sistema.
  SELECT COUNT(1) INTO busca_pk2 FROM PROCESO WHERE id_proceso = pProceso;
  -- Si no existe
  RSP := 'RESULTADO: ERROR: El proceso no existe.';
  IF busca_pk2 = 0 THEN RAISE MI_EXC; END IF;
     
  -- INSERCION: Comprobado todo, tratamos de insertar el registro.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  INSERT INTO ESTUDIOPROCESO(id_analisis, id_proceso, f_estudioproceso) VALUES (pAnalisis, pProceso, pFecha);
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Cambiamos el estado de ha_sido_estudiado del proceso indicado. 
  UPDATE PROCESO SET ha_sido_analizado_proceso = 1 WHERE id_proceso = pProceso;
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_ESTUDIOPROCESO;


PROCEDURE BAJA_ESTUDIOPROCESO (
   pAnalisis IN ESTUDIOPROCESO.id_analisis%TYPE, 
   pProceso IN ESTUDIOPROCESO.id_proceso%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk	NUMBER(1);
   
   BEGIN
    PAR_IN := 'Analisis: '||pAnalisis||', Proceso: '||pProceso;
	PROC := 'PKG_ABM.BAJA_ESTUDIOPROCESO';	
	
    -- Comprobamos que no se esta dejando ningun valor de la PK vacio.
    RSP := 'RESULTADO: ERROR: El identificador del analisis no puede ser nulo.';
    IF pAnalisis IS NULL THEN RAISE MI_EXC; END IF;
	RSP := 'RESULTADO: ERROR: El identificador del proceso no puede ser nulo.';
    IF pProceso IS NULL THEN RAISE MI_EXC; END IF;

	-- Comprobamos que el par de id proporcionados corresponde a un estudio de proceso registrado en el sistema.
	SELECT COUNT(1) INTO busca_pk FROM ESTUDIOPROCESO WHERE id_analisis = pAnalisis AND id_proceso = pProceso;
	
	IF busca_pk <> 0 THEN 
    -- SI EXISTE, LA ELIMINAMOS; SOLO PUEDE HABER UNA PORQUE ES PK
		RSP := 'RESULTADO: OK';
		DELETE FROM ESTUDIOPROCESO WHERE id_analisis = pAnalisis AND id_proceso = pProceso;
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: El registro de estudio de proceso no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_ESTUDIOPROCESO;


PROCEDURE MODIFICA_ESTUDIOPROCESO ( 
   pAnalisis IN ESTUDIOPROCESO.id_analisis%TYPE, 
   pProceso IN ESTUDIOPROCESO.id_proceso%TYPE, 
   pFecha IN ESTUDIOPROCESO.f_estudioproceso%TYPE, 
   RSP OUT VARCHAR2) AS
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
   busca_pk NUMBER(1);
   busca_pk2 NUMBER(1);
   fecha_inicio_analisis	DATE;
   fecha_fin_analisis	DATE;
   
  BEGIN
  PAR_IN := 'Analisis: '||pAnalisis||', Proceso: '||pProceso||', Fecha del estudio: '||pFecha;
  PROC := 'PKG_ABM.MODIFICA_ESTUDIOPROCESO';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El analisis no puede ser nulo.';
  IF pAnalisis IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El proceso no puede ser nulo.';
  IF pProceso IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha del estudio no puede ser nula.';
  IF pFecha IS NULL THEN RAISE MI_EXC; END IF;
  
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- La fecha de estudio de un proceso durante un análisis de vulnerabilidades siempre será mayor o igual a la fecha de inicio del análisis, y menor o igual a la fecha de fin del análisis (si está registrada).
  -- Comprobamos que el id proporcionado corresponde a un analisis registrado en el sistema.
  SELECT COUNT(1) INTO busca_pk FROM ANALISIS WHERE id_analisis = pAnalisis;
  IF busca_pk <> 0 THEN 
	-- SI EXISTE, OBTENEMOS LAS FECHAS QUE NOS INTERESAN
	SELECT f_inicio_analisis INTO fecha_inicio_analisis FROM ANALISIS WHERE id_analisis = pAnalisis;
	SELECT f_fin_analisis INTO fecha_fin_analisis FROM ANALISIS WHERE id_analisis = pAnalisis;
  ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: El analisis no existe.';
		RAISE MI_EXC;
	END IF;
	
  -- Comprobamos las fechas	
  IF pFecha > fecha_inicio_analisis THEN 
	IF fecha_fin_analisis IS NOT NULL THEN 
		IF pFecha > fecha_fin_analisis THEN 
		-- La fecha de estudio no coincide con la fecha de fin del analisis
			RSP := 'RESULTADO: ERROR: La fecha de estudio no coincide con la fecha de fin del analisis.';
			RAISE MI_EXC;
		END IF;
	END IF;
  ELSE 
  -- La fecha de estudio no coincide con la fecha de inicio del analisis
	RSP := 'RESULTADO: ERROR: La fecha de estudio no coincide con la fecha de inicio del analisis.';
	RAISE MI_EXC;
  END IF;
  
  -- Comprobamos que el id proporcionado corresponde a un proceso registrado en el sistema.
  SELECT COUNT(1) INTO busca_pk2 FROM PROCESO WHERE id_proceso = pProceso;
  -- Si no existe
  RSP := 'RESULTADO: ERROR: El proceso no existe.';
  IF busca_pk2 = 0 THEN RAISE MI_EXC; END IF;
       
   -- MODIFICACION: Comprobado todo, tratamos de modificar el analisis con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE ESTUDIOPROCESO SET f_estudioproceso = pFecha WHERE id_analisis = pAnalisis AND id_proceso = pProceso;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  --  Cambiamos el estado de ha_sido_estudiado del proceso indicado. No hace falta, ya que al ser una modificacion ya lo habriamos cambiado anteriormente, pero por coherencia.
  UPDATE PROCESO SET ha_sido_analizado_proceso = 1 WHERE id_proceso = pProceso;
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_ESTUDIOPROCESO;
 
 
/*************************
***** INCUMPLIMIENTO *****
**************************/
 
PROCEDURE ALTA_INCUMPLIMIENTO ( 
   pFecha IN INCUMPLIMIENTO.f_deteccion_incumplimiento%TYPE, 
   pAuditoria IN INCUMPLIMIENTO.id_auditoria%TYPE, 
   pEmpleado IN INCUMPLIMIENTO.id_empleado%TYPE, 
   pPolitica IN INCUMPLIMIENTO.id_politica%TYPE, 
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
   busca_pk 	NUMBER(1);
   fecha_ini	DATE;
   fecha_fin	DATE;
      
  BEGIN
  PAR_IN := 'Fecha de deteccion: '||pFecha||', Auditoria: '||pAuditoria||', Empleado: '||pEmpleado||', Politica: '||pPolitica;
  PROC := 'PKG_ABM.ALTA_INCUMPLIMIENTO';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: La fecha de deteccion del incumplimiento no puede ser nulo.';
  IF pFecha IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La auditoria del incumplimiento no puede ser nulo.';
  IF pAuditoria IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El empleado del incumplimiento no puede ser nulo.';
  IF pEmpleado IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La politica del incumplimiento no puede ser nulo.';
  IF pPolitica IS NULL THEN RAISE MI_EXC; END IF;
  
  -- Comprobamos que los id proporcionados corresponden a registros en el sistema.
  -- Al ser FK, lo comprobaria el SGBD, pero asi personalizamos el mensaje en el log de operaciones en caso de error.
  SELECT COUNT(1) INTO busca_pk FROM AUDITORIA WHERE id_auditoria = pAuditoria;
  RSP := 'RESULTADO: ERROR: La auditoria del incumplimiento no existe.';
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  SELECT COUNT(1) INTO busca_pk FROM EMPLEADO WHERE id_empleado = pEmpleado;
  RSP := 'RESULTADO: ERROR: El empleado del incumplimiento no existe.';
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  SELECT COUNT(1) INTO busca_pk FROM POLITICA WHERE id_politica = pPolitica;
  RSP := 'RESULTADO: ERROR: La politica del incumplimiento no existe.';
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- La fecha de detección de un incumplimiento debe ser anterior o igual a la fecha actual
  RSP := 'RESULTADO: ERROR: La fecha de deteccion de un incumplimiento debe ser anterior o igual a la fecha actual.';
  IF pFecha > SYSDATE THEN RAISE MI_EXC; END IF;
  
  -- La fecha de detección de un incumplimiento debe estar entre las fechas de inicio y de finalización de la auditoria en la que se ha detectado.
  SELECT f_inicio_auditoria INTO fecha_ini FROM AUDITORIA WHERE id_auditoria = pAuditoria;
  SELECT f_fin_auditoria INTO fecha_fin FROM AUDITORIA WHERE id_auditoria = pAuditoria;
  IF (pFecha < fecha_ini OR pFecha > fecha_fin) THEN RAISE MI_EXC; END IF;
  
  -- INSERCION: Comprobado todo, tratamos de modificar el registro.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  INSERT INTO INCUMPLIMIENTO(f_deteccion_incumplimiento, id_auditoria, id_empleado, id_politica) VALUES (pFecha, pAuditoria, pEmpleado, pPolitica);
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Al dar de alta o de baja un incumplimiento (no se admite la modificacion de la auditoria, el empleado o la politica asociados al incumplimiento), actualizamos el campo num_incumplimientos de POLITICA
  TFG.PKG_FUNCIONAMIENTO.ACTUALIZAR_INCUMPLIMIENTOS_DE_POLITICA(pPolitica, RSP);
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_INCUMPLIMIENTO;


PROCEDURE BAJA_INCUMPLIMIENTO (
   pId IN INCUMPLIMIENTO.id_incumplimiento%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk	NUMBER(1);
   vPolitica	NUMBER(10);
   
   BEGIN
    PAR_IN := 'Id Incumplimiento: '||pId;
	PROC := 'PKG_ABM.BAJA_INCUMPLIMIENTO';	
	
    -- Comprobamos que no se esta dejando ningun valor de la PK vacio.
    RSP := 'RESULTADO: ERROR: El identificador del incumplimiento no puede ser nulo.';
    IF pId IS NULL THEN RAISE MI_EXC; END IF;

	-- Comprobamos que el id proporcionado corresponde a un incumplimiento registrado en el sistema.
	SELECT COUNT(1) INTO busca_pk FROM INCUMPLIMIENTO WHERE id_incumplimiento = pId;
	
	IF busca_pk <> 0 THEN 
    -- SI EXISTE, LO ELIMINAMOS; SOLO PUEDE HABER UNO PORQUE ES PK
	-- Pero primero obtenemos el ID de politica asociado
		SELECT id_politica INTO vPolitica FROM INCUMPLIMIENTO WHERE id_incumplimiento = pId;
		RSP := 'RESULTADO: OK';
		DELETE FROM INCUMPLIMIENTO WHERE id_incumplimiento = pId;
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
	-- Al dar de alta o de baja un incumplimiento (no se admite la modificacion de la auditoria, el empleado o la politica asociados al incumplimiento), actualizamos el campo num_incumplimientos de POLITICA
		TFG.PKG_FUNCIONAMIENTO.ACTUALIZAR_INCUMPLIMIENTOS_DE_POLITICA(vPolitica, RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: El incumplimiento no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_INCUMPLIMIENTO;


PROCEDURE MODIFICA_INCUMPLIMIENTO ( 
   pId IN INCUMPLIMIENTO.id_incumplimiento%TYPE, 
   pFecha IN INCUMPLIMIENTO.f_deteccion_incumplimiento%TYPE, 
   RSP OUT VARCHAR2) AS
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
    
   busca_pk 	NUMBER(1);
   fecha_ini	DATE;
   fecha_fin	DATE;
   
   vAuditoria	NUMBER(10);
      
  BEGIN
  -- NOTA: no se admite la modificacion de la auditoria, el empleado o la politica asociados al incumplimiento. En caso de error, se eliminara el incumplimiento y se registrara de nuevo correctamente.
  
  PAR_IN := 'Identificador: '||pId||', Fecha de deteccion: '||pFecha;
  PROC := 'PKG_ABM.MODIFICA_INCUMPLIMIENTO';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El incumplimiento no puede ser nulo.';
  IF pId IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha de deteccion del incumplimiento no puede ser nulo.';
  IF pFecha IS NULL THEN RAISE MI_EXC; END IF;
  
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- La fecha de deteccion de un incumplimiento debe ser anterior o igual a la fecha actual
  RSP := 'RESULTADO: ERROR: La fecha de detección de un incumplimiento debe ser anterior o igual a la fecha actual.';
  IF pFecha > SYSDATE THEN RAISE MI_EXC; END IF;
  
  SELECT id_auditoria INTO vAuditoria FROM INCUMPLIMIENTO WHERE id_incumplimiento = pId; 
  
  -- La fecha de detección de un incumplimiento debe estar entre las fechas de inicio y de finalización de la auditoria en la que se ha detectado.
  SELECT f_inicio_auditoria INTO fecha_ini FROM AUDITORIA WHERE id_auditoria = vAuditoria;
  SELECT f_fin_auditoria INTO fecha_fin FROM AUDITORIA WHERE id_auditoria = vAuditoria;
  IF (pFecha < fecha_ini OR pFecha > fecha_fin) THEN RAISE MI_EXC; END IF;
  
   -- MODIFICACION: Comprobado todo, tratamos de modificar el analisis con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE INCUMPLIMIENTO SET f_deteccion_incumplimiento = pFecha WHERE id_incumplimiento = pId;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_INCUMPLIMIENTO;
 
 
 
/***********************************
***** INCUMPLIMIENTOSANYODEPTO *****
************************************/

-- Las actualizaciones de esta tabla se realizarán desde procedimientos del paquete PKG_ESTADISTICAS

/**************************************
***** INCUMPLIMIENTOSEMPLEADOANYO *****
***************************************/

-- Las actualizaciones de esta tabla se realizarán desde procedimientos del paquete PKG_ESTADISTICAS
 
/*************************
***** LOGOPERACIONES *****
**************************/

-- Todas las inserciones en esta tabla se realizaran desde procedimientos almacenados, y no habrá bajas de registros, ni modificaciones.

/*******************
***** MUESTREO *****
********************/

PROCEDURE ALTA_MUESTREO ( 
   pPolitica IN MUESTREO.id_politica%TYPE, 
   pAuditoria IN MUESTREO.id_auditoria%TYPE,
   pEmpleado IN MUESTREO.id_empleado%TYPE,
   pF_muestreo IN MUESTREO.f_muestreo%TYPE,
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
      
   busca_pk 	NUMBER(1);
   fecha_ini	DATE;
   fecha_fin	DATE;
   
  BEGIN
  PAR_IN := 'Politica: '||pPolitica||', Auditoria: '||pAuditoria||', Empleado: '||pEmpleado||', Fecha del muestreo: '||pF_muestreo;
  PROC := 'PKG_ABM.ALTA_MUESTREO';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: La politica a la que hace referencia el muestreo no puede ser nulo.';
  IF pPolitica IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La auditoria a la que hace referencia el muestreo no puede ser nulo.';
  IF pAuditoria IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El empleado al que hace referencia el muestreo no puede ser nulo.';
  IF pEmpleado IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha del muestreo no puede ser nulo.';
  IF pF_muestreo IS NULL THEN RAISE MI_EXC; END IF;
  
  -- Comprobamos que los id proporcionados corresponden a registros en el sistema.
  -- Al ser FK, lo comprobaria el SGBD, pero asi personalizamos el mensaje en el log de operaciones en caso de error.
  SELECT COUNT(1) INTO busca_pk FROM AUDITORIA WHERE id_auditoria = pAuditoria;
  RSP := 'RESULTADO: ERROR: La auditoria del muestreo no existe.';
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  SELECT COUNT(1) INTO busca_pk FROM EMPLEADO WHERE id_empleado = pEmpleado;
  RSP := 'RESULTADO: ERROR: El empleado del muestreo no existe.';
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  SELECT COUNT(1) INTO busca_pk FROM POLITICA WHERE id_politica = pPolitica;
  RSP := 'RESULTADO: ERROR: La politica del muestreo no existe.';
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- La fecha del muestreo debe estar entre las fechas de inicio y de finalización de la auditoria en la que se ha tomado.
  SELECT f_inicio_auditoria INTO fecha_ini FROM AUDITORIA WHERE id_auditoria = pAuditoria;
  SELECT f_fin_auditoria INTO fecha_fin FROM AUDITORIA WHERE id_auditoria = pAuditoria;
  -- Como la fecha de fin puede o no estar registrada (no es obligatoria, ya que en principio se desconocera al principio de la auditoria), comprobamos la fecha de inicio, y si esta es correcta comparamos con la de fin, si existe.
  RSP := 'RESULTADO: ERROR: La fecha del muestreo no puede ser anterior al inicio de la auditoria.';
  IF (pF_muestreo < fecha_ini) THEN RAISE MI_EXC; 
  ELSE 
	IF fecha_fin IS NOT NULL THEN
		RSP := 'RESULTADO: ERROR: La fecha del muestreo no puede ser posterior a la finalizacion de la auditoria.';
		IF (pF_muestreo > fecha_fin ) THEN RAISE MI_EXC; END IF;
	END IF;
  END IF;
  
  -- INSERCION: Comprobado todo, tratamos de modificar el registro.
  -- Comprobamos previamente que no exista ya un muestreo con los mismos datos.
  SELECT COUNT(1) INTO busca_pk FROM MUESTREO WHERE id_auditoria = pAuditoria AND id_empleado = pEmpleado AND id_politica = pPolitica;
  -- Ya existe un muestreo con los mismos datos.
  RSP := 'RESULTADO: ERROR: El muestreo indicado ya existe.';
  IF busca_pk <> 0 THEN RAISE MI_EXC;
  ELSE
	RSP := 'RESULTADO: OK';
	INSERT INTO MUESTREO(id_politica, id_auditoria, id_empleado, f_muestreo) VALUES (pPolitica, pAuditoria, pEmpleado, pF_muestreo);
	-- Registramos la actividad en el LOGOPERACIONES
	INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  END IF; 
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso. 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_MUESTREO;


PROCEDURE BAJA_MUESTREO (
   pPolitica IN MUESTREO.id_politica%TYPE, 
   pAuditoria IN MUESTREO.id_auditoria%TYPE,
   pEmpleado IN MUESTREO.id_empleado%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk	NUMBER(1);
   
   BEGIN
    PAR_IN := 'Politica: '||pPolitica||', Auditoria: '||pAuditoria||', Empleado: '||pEmpleado;
	PROC := 'PKG_ABM.BAJA_MUESTREO';	
	
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: La politica a la que hace referencia el muestreo no puede ser nulo.';
  IF pPolitica IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La auditoria a la que hace referencia el muestreo no puede ser nulo.';
  IF pAuditoria IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El empleado al que hace referencia el muestreo no puede ser nulo.';
  IF pEmpleado IS NULL THEN RAISE MI_EXC; END IF;
  
  -- Comprobamos que los id proporcionados identifican a un registro de muestreo en el sistema.
  -- Al ser PK, lo comprobaria el SGBD, pero asi personalizamos el mensaje en el log de operaciones en caso de error.
  SELECT COUNT(1) INTO busca_pk FROM MUESTREO WHERE id_auditoria = pAuditoria AND id_empleado = pEmpleado AND id_politica = pPolitica;
  IF busca_pk <> 0 THEN 
	-- SI EXISTE, LO ELIMINAMOS; SOLO PUEDE HABER UNO PORQUE ES PK
		RSP := 'RESULTADO: OK';
		DELETE FROM MUESTREO WHERE id_auditoria = pAuditoria AND id_empleado = pEmpleado AND id_politica = pPolitica;
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: El muestreo indicado no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_MUESTREO;


PROCEDURE MODIFICA_MUESTREO ( 
   pPolitica IN MUESTREO.id_politica%TYPE, 
   pAuditoria IN MUESTREO.id_auditoria%TYPE,
   pEmpleado IN MUESTREO.id_empleado%TYPE,
   pF_muestreo IN MUESTREO.f_muestreo%TYPE,
   RSP OUT VARCHAR2) AS
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
         
   busca_pk 	NUMBER(1);
   fecha_ini	DATE;
   fecha_fin	DATE;
   
  BEGIN
  PAR_IN := 'Politica: '||pPolitica||', Auditoria: '||pAuditoria||', Empleado: '||pEmpleado||', Fecha del muestreo: '||pF_muestreo;
  PROC := 'PKG_ABM.MODIFICA_MUESTREO';
  
-- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: La politica a la que hace referencia el muestreo no puede ser nulo.';
  IF pPolitica IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La auditoria a la que hace referencia el muestreo no puede ser nulo.';
  IF pAuditoria IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El empleado al que hace referencia el muestreo no puede ser nulo.';
  IF pEmpleado IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha del muestreo no puede ser nulo.';
  IF pF_muestreo IS NULL THEN RAISE MI_EXC; END IF;
  
  -- Comprobamos que los id proporcionados identifican a un registro de muestreo en el sistema.
  -- Al ser PK, lo comprobaria el SGBD, pero asi personalizamos el mensaje en el log de operaciones en caso de error.
  SELECT COUNT(1) INTO busca_pk FROM MUESTREO WHERE id_auditoria = pAuditoria AND id_empleado = pEmpleado AND id_politica = pPolitica;
  -- No existe.
  RSP := 'RESULTADO: ERROR: El muestreo indicado no existe.';
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
		
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- La fecha del muestreo debe estar entre las fechas de inicio y de finalización de la auditoria en la que se ha tomado.
  SELECT f_inicio_auditoria INTO fecha_ini FROM AUDITORIA WHERE id_auditoria = pAuditoria;
  SELECT f_fin_auditoria INTO fecha_fin FROM AUDITORIA WHERE id_auditoria = pAuditoria;
  -- Como la fecha de fin puede o no estar registrada (no es obligatoria, ya que en principio se desconocera al principio de la auditoria), comprobamos la fecha de inicio, y si esta es correcta comparamos con la de fin, si existe.
  RSP := 'RESULTADO: ERROR: La fecha del muestreo no puede ser anterior al inicio de la auditoria.';
  IF (pF_muestreo < fecha_ini) THEN RAISE MI_EXC; 
  ELSE 
	IF fecha_fin IS NOT NULL THEN
		RSP := 'RESULTADO: ERROR: La fecha del muestreo no puede ser posterior a la finalizacion de la auditoria.';
		IF (pF_muestreo > fecha_fin ) THEN RAISE MI_EXC; END IF;
	END IF;
  END IF;
  
  -- MODIFICACION: Comprobado todo, tratamos de modificar el muestreo con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE MUESTREO SET f_muestreo = pF_muestreo WHERE id_auditoria = pAuditoria AND id_empleado = pEmpleado AND id_politica = pPolitica;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
    
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso. 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_MUESTREO;

/********************
 ***** POLITICA *****
 ********************/
 
 PROCEDURE ALTA_POLITICA ( 
   pNombre IN POLITICA.nombre_politica%TYPE,
   pURL IN POLITICA.url_politica%TYPE,
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
      
   busca_pk 	NUMBER(1);
   fecha_ini	DATE;
   fecha_fin	DATE;
   
  BEGIN
  PAR_IN := 'Nombre: '||pNombre||', URL Documentacion: '||pURL;
  PROC := 'PKG_ABM.ALTA_POLITICA';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: La nombre de politica no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
  -- El campo num_incumplimientos no acepta  nulos, pero se inicializa por defecto a 0, y se actualiza automáticamente.
  
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten.
  -- En este caso, ninguna.
  
  -- INSERCION: Comprobado todo, tratamos de modificar el registro.
  RSP := 'RESULTADO: OK';
  INSERT INTO POLITICA(nombre_politica, url_politica) VALUES (pNombre, pURL);
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso. 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_POLITICA;

PROCEDURE BAJA_POLITICA (
   pId IN POLITICA.id_politica%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk	NUMBER(1);
   
   BEGIN
    PAR_IN := 'Politica: '||pId;
	PROC := 'PKG_ABM.BAJA_POLITICA';	
	
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de politica no puede ser nulo.';
  IF pId IS NULL THEN RAISE MI_EXC; END IF;
  
  -- Comprobamos que el id proporcionado identifica a una politica en el sistema.
  -- Al ser PK, lo comprobaria el SGBD, pero asi personalizamos el mensaje en el log de operaciones en caso de error.
  SELECT COUNT(1) INTO busca_pk FROM POLITICA WHERE id_politica = pId;
  IF busca_pk <> 0 THEN 
	-- SI EXISTE, LA ELIMINAMOS; SOLO PUEDE HABER UNA PORQUE ES PK
		RSP := 'RESULTADO: OK';
		DELETE FROM POLITICA WHERE id_politica = pId;
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: La politica indicada no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_POLITICA;


PROCEDURE MODIFICA_POLITICA ( 
   pId IN POLITICA.id_politica%TYPE, 
   pNombre IN POLITICA.nombre_politica%TYPE,
   pURL IN POLITICA.url_politica%TYPE,
   RSP OUT VARCHAR2) AS
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
         
   busca_pk 	NUMBER(1);
   
  BEGIN
  PAR_IN := 'Politica: '||pId||', Nombre: '||pNombre||', URL Documentacion: '||pURL;
  PROC := 'PKG_ABM.MODIFICA_POLITICA';
  
-- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de politica no puede ser nulo.';
  IF pId IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El nombre de la politica no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
  
  -- Comprobamos que el id proporcionado identifica a una politica registrada en el sistema.
  -- Al ser PK, lo comprobaria el SGBD, pero asi personalizamos el mensaje en el log de operaciones en caso de error.
  SELECT COUNT(1) INTO busca_pk FROM POLITICA WHERE id_politica = pId;
  RSP := 'RESULTADO: ERROR: La politica indicada no existe.';
  -- Si no existe
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  	
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- En este caso, ninguna.
  
  -- MODIFICACION: Comprobado todo, tratamos de modificar la politica con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE POLITICA SET nombre_politica = pNombre, url_politica = pURL WHERE id_politica = pId;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
    
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso. 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_POLITICA;
 
/******************
***** PROCESO *****
*******************/
 
 PROCEDURE ALTA_PROCESO ( 
   pNombre IN PROCESO.nombre_proceso%TYPE,
   pURL IN PROCESO.url_proceso%TYPE,
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
      
   busca_pk 	NUMBER(1);
   fecha_ini	DATE;
   fecha_fin	DATE;
   
  BEGIN
  PAR_IN := 'Nombre: '||pNombre||', URL Documentacion: '||pURL;
  PROC := 'PKG_ABM.ALTA_PROCESO';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El nombre del proceso no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
  -- Los campos ha_sido_analizado_proceso y num_vuln_detectadas no aceptan  nulos, pero se inicializan por defecto a 0, y se actualizan automáticamente.
  
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten.
  -- En este caso, ninguna.
  
  -- INSERCION: Comprobado todo, tratamos de modificar el registro.
  RSP := 'RESULTADO: OK';
  INSERT INTO PROCESO(nombre_proceso, url_proceso) VALUES (pNombre, pURL);
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso. 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_PROCESO;

PROCEDURE BAJA_PROCESO (
   pId IN PROCESO.id_proceso%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   busca_pk	NUMBER(1);
   
   BEGIN
    PAR_IN := 'Proceso: '||pId;
	PROC := 'PKG_ABM.BAJA_PROCESO';	
	
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de proceso no puede ser nulo.';
  IF pId IS NULL THEN RAISE MI_EXC; END IF;
  
  -- Comprobamos que el id proporcionado identifica a un proceso en el sistema.
  -- Al ser PK, lo comprobaria el SGBD, pero asi personalizamos el mensaje en el log de operaciones en caso de error.
  SELECT COUNT(1) INTO busca_pk FROM PROCESO WHERE id_proceso = pId;
  IF busca_pk <> 0 THEN 
	-- SI EXISTE, LO ELIMINAMOS; SOLO PUEDE HABER UNO PORQUE ES PK
		RSP := 'RESULTADO: OK';
		DELETE FROM PROCESO WHERE id_proceso = pId;
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: El proceso indicado no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_PROCESO;


PROCEDURE MODIFICA_PROCESO ( 
   pId IN PROCESO.id_proceso%TYPE, 
   pNombre IN PROCESO.nombre_proceso%TYPE,
   pURL IN PROCESO.url_proceso%TYPE,
   RSP OUT VARCHAR2) AS
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
         
   busca_pk 	NUMBER(1);
   
  BEGIN
  PAR_IN := 'Proceso: '||pId||', Nombre: '||pNombre||', URL Documentacion: '||pURL;
  PROC := 'PKG_ABM.MODIFICA_PROCESO';
  
-- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de proceso no puede ser nulo.';
  IF pId IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El nombre del proceso no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;
  
  -- Comprobamos que el id proporcionado identifica a un proceso registrado en el sistema.
  -- Al ser PK, lo comprobaria el SGBD, pero asi personalizamos el mensaje en el log de operaciones en caso de error.
  SELECT COUNT(1) INTO busca_pk FROM PROCESO WHERE id_proceso = pId;
  -- Si no existe
  RSP := 'RESULTADO: ERROR: El proceso indicado no existe.';
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  	
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten
  -- En este caso, ninguna.
  
  -- MODIFICACION: Comprobado todo, tratamos de modificar la politica con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE PROCESO SET nombre_proceso = pNombre, url_proceso = pURL WHERE id_proceso = pId;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
    
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso. 
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_PROCESO;
 
/*****************
***** SESION *****
******************/
 
 PROCEDURE ALTA_SESION ( 
   pTitulo IN SESION.titulo_sesion%TYPE,
   pModalidad IN SESION.modalidad_sesion%TYPE,
   pF_inicio IN SESION.f_inicio_sesion%TYPE,
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
  BEGIN
  PAR_IN := 'Titulo Sesion: '||pTitulo||', Modalidad: '||pModalidad||', Fecha de inicio: '||pF_inicio;
  PROC := 'PKG_ABM.ALTA_SESION';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El titulo de la sesion no puede ser nulo.';
  IF pTitulo IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La modalidad de la sesion no puede ser nulo.';
  IF pModalidad IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha de inicio de la sesion no puede ser nulo.';
  IF pF_inicio IS NULL THEN RAISE MI_EXC; END IF;
  
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten.
  -- Los valores admitidos para la modalidad de una sesion son PRESENCIAL y TELEMATICA.
  RSP := 'RESULTADO: ERROR: Los valores admitidos para la modalidad de una sesion son PRESENCIAL y TELEMATICA.';
  IF (UPPER(pModalidad) <> 'PRESENCIAL' AND UPPER(pModalidad) <> 'TELEMATICA') THEN RAISE MI_EXC; END IF;
  
  -- INSERCION: Comprobado todo, tratamos de modificar el registro.
  RSP := 'RESULTADO: OK';
  INSERT INTO SESION(titulo_sesion, modalidad_sesion, f_inicio_sesion) VALUES (pTitulo, UPPER(pModalidad), pF_inicio);
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- TODO
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_SESION;

PROCEDURE BAJA_SESION (
   pId IN SESION.id_sesion%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
   busca_pk	NUMBER(1);
   
   BEGIN
    PAR_IN := 'Sesion: '||pId;
	PROC := 'PKG_ABM.BAJA_SESION';	
	
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de sesion no puede ser nulo.';
  IF pId IS NULL THEN RAISE MI_EXC; END IF;
  
  -- Comprobamos que el id proporcionado identifica a una sesion registrada en el sistema.
  -- El SGBD lo comprobaria, pero asi personalizamos el mensaje en el log de operaciones en caso de error.
  SELECT COUNT(1) INTO busca_pk FROM SESION WHERE id_sesion = pId;
  IF busca_pk <> 0 THEN 
	-- SI EXISTE, LA ELIMINAMOS; SOLO PUEDE HABER UNA PORQUE ES PK
		RSP := 'RESULTADO: OK';
		DELETE FROM SESION WHERE id_sesion = pId;
	-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: La sesion indicada no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_SESION;

PROCEDURE MODIFICA_SESION ( 
   pId IN SESION.id_sesion%TYPE, 
   pTitulo IN SESION.titulo_sesion%TYPE,
   pModalidad IN SESION.modalidad_sesion%TYPE,
   pF_inicio IN SESION.f_inicio_sesion%TYPE,
   RSP OUT VARCHAR2) AS
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
         
   busca_pk 	NUMBER(1);
   
  BEGIN
  PAR_IN := 'Sesion: '||pId||', Titulo Sesion: '||pTitulo||', Modalidad: '||pModalidad||', Fecha de inicio: '||pF_inicio;
  PROC := 'PKG_ABM.MODIFICA_SESION';
  
 -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El titulo de la sesion no puede ser nulo.';
  IF pTitulo IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La modalidad de la sesion no puede ser nulo.';
  IF pModalidad IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha de inicio de la sesion no puede ser nulo.';
  IF pF_inicio IS NULL THEN RAISE MI_EXC; END IF;
  
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten.
  -- Los valores admitidos para la modalidad de una sesion son PRESENCIAL y TELEMATICA.
  RSP := 'RESULTADO: ERROR: Los valores admitidos para la modalidad de una sesion son PRESENCIAL y TELEMATICA.';
  IF (UPPER(pModalidad) <> 'PRESENCIAL' AND UPPER(pModalidad) <> 'TELEMATICA') THEN RAISE MI_EXC; END IF;
  
  -- Comprobamos que el id proporcionado identifica a una sesion registrado en el sistema.
  -- Al ser PK, lo comprobaria el SGBD, pero asi personalizamos el mensaje en el log de operaciones en caso de error.
  SELECT COUNT(1) INTO busca_pk FROM SESION WHERE id_sesion = pId;
  -- Si no existe
  RSP := 'RESULTADO: ERROR: La sesion indicada no existe.';
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  	
  -- MODIFICACION: Comprobado todo, tratamos de modificar la politica con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE SESION SET titulo_sesion = pTitulo, modalidad_sesion = pModalidad, f_inicio_sesion = f_inicio_sesion WHERE id_sesion = pId;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
    
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- TODO
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_SESION;
 
/*************************
***** VULNERABILIDAD *****
**************************/

 PROCEDURE ALTA_VULNERABILIDAD ( 
   pAnalisis IN VULNERABILIDAD.id_analisis%TYPE,
   pProceso IN VULNERABILIDAD.id_proceso%TYPE,
   pF_deteccion IN VULNERABILIDAD.f_deteccion_vuln%TYPE,
   pEstado IN VULNERABILIDAD.id_estado_vuln%TYPE,
   pCritica IN VULNERABILIDAD.es_vuln_critica%TYPE,
   pNombre IN VULNERABILIDAD.nombre_vuln%TYPE,
   pURL IN VULNERABILIDAD.url_vuln%TYPE,
   RSP OUT VARCHAR2) AS 
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
   busca_pk	NUMBER(1);
   f_ini 	DATE;
   f_fin	DATE;
   
  BEGIN
  PAR_IN := 'Analisis: '||pAnalisis||', Proceso: '||pProceso||', Fecha de deteccion: '||pF_deteccion||', ID estado: '||pEstado||', Es Critica: '||pCritica||', Nombre: '||pNombre||', URL Documentacion: '||pURL;
  PROC := 'PKG_ABM.ALTA_VULNERABILIDAD';
  
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El analisis asociado al descubrimiento de la vulnerabilidad no puede ser nulo.';
  IF pAnalisis IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El proceso sobre el que se ha descubierto la vulnerabilidad no puede ser nulo.';
  IF pProceso IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha de deteccion de la vulnerabilidad no puede ser nulo.';
  IF pF_deteccion IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El estado de la vulnerabilidad no puede ser nulo.';
  IF pEstado IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El indicador de si la vulnerabilidad es critica o no no puede ser nulo.';
  IF pCritica IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El nombre de la vulnerabilidad no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;

  -- Comprobamos que los id proporcionados corresponden a registros en el sistema.
  -- Al ser FK, lo comprobaria el SGBD, pero asi personalizamos el mensaje en el log de operaciones en caso de error.
  RSP := 'RESULTADO: ERROR: El analisis indicado no existe.';
  SELECT COUNT(1) INTO busca_pk FROM ANALISIS WHERE id_analisis = pAnalisis;
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El proceso indicado no existe.';
  SELECT COUNT(1) INTO busca_pk FROM PROCESO WHERE id_proceso = pProceso;
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  
  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten.
  -- Los estados por los que puede pasar una vulnerabilidad son los registrados en la tabla ESTADOSVULN.
  -- Se podrian añadir nuevos estados, pero se inicializan al cargar las tablas maestras. 
  RSP := 'RESULTADO: ERROR: El estado indicado no existe.';
  SELECT COUNT(1) INTO busca_pk FROM ESTADOVULN WHERE id_estado_vuln = pEstado;
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  
  -- La fecha de detección de una vulnerabilidad siempre será anterior o igual a la fecha actual.
  RSP := 'RESULTADO: ERROR: La fecha de deteccion de la vulnerabilidad no puede estar en el futuro.';
  IF (pF_deteccion > SYSDATE) THEN RAISE MI_EXC; END IF;
  
  -- La fecha de detección de una vulnerabilidad deberá ser siempre posterior o igual a la fecha de inicio del analisis, y anterior o igual a la fecha de fin de dicho analisis (si se ha registrado).
  
  SELECT f_inicio_analisis INTO f_ini FROM ANALISIS WHERE id_analisis = pAnalisis;
  SELECT f_fin_analisis INTO f_fin FROM ANALISIS WHERE id_analisis = pAnalisis;
  
  -- Como la fecha de fin puede o no estar registrada (no es obligatoria, ya que en principio se desconocera al principio del analisis), comprobamos la fecha de inicio, y si esta es correcta comparamos con la de fin, si existe.
  RSP := 'RESULTADO: ERROR: La fecha de la deteccion no puede ser anterior al inicio del analisis.';
  IF (pF_deteccion < f_ini) THEN RAISE MI_EXC; 
  ELSE 
	IF f_fin IS NOT NULL THEN
		RSP := 'RESULTADO: ERROR: La fecha del muestreo no puede ser posterior a la finalizacion del analisis.';
		IF (pF_deteccion > f_fin ) THEN RAISE MI_EXC; END IF;
	END IF;
  END IF;
  
  -- INSERCION: Comprobado todo, tratamos de modificar el registro.
  RSP := 'RESULTADO: OK';
  INSERT INTO VULNERABILIDAD(id_analisis, id_proceso, f_deteccion_vuln, id_estado_vuln, es_vuln_critica, nombre_vuln, url_vuln) VALUES (pAnalisis, pProceso, pF_deteccion, pEstado, pCritica, pNombre, pURL);
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
  
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Al dar de alta o de baja una vulnerabilidad (no se permite modificar el proceso ni el analisis asociados a la vulnerabilidad), se recalcula desde el mismo procedimiento ABM el número total de vulnerabilidades del proceso (num_vuln_detectadas)
  TFG.PKG_FUNCIONAMIENTO.ACTUALIZAR_VULNERABILIDADES_DE_PROCESO(pProceso,RSP);
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END ALTA_VULNERABILIDAD;

PROCEDURE BAJA_VULNERABILIDAD (
   pId IN VULNERABILIDAD.id_vulnerabilidad%TYPE, 
   RSP OUT VARCHAR2) AS 
     
   MI_EXC   EXCEPTION;
   PAR_IN   VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
   busca_pk	NUMBER(1);
   vProceso	NUMBER(10);
   
   BEGIN
    PAR_IN := 'Vulnerabilidad: '||pId;
	PROC := 'PKG_ABM.BAJA_VULNERABILIDAD';	
	
  -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de vulnerabilidad no puede ser nulo.';
  IF pId IS NULL THEN RAISE MI_EXC; END IF;
  
  -- Comprobamos que el id proporcionado identifica a una vulnerabilidad registrada en el sistema.
  -- El SGBD lo comprobaria, pero asi personalizamos el mensaje en el log de operaciones en caso de error.
  SELECT COUNT(1) INTO busca_pk FROM VULNERABILIDAD WHERE id_vulnerabilidad = pId;
  IF busca_pk <> 0 THEN 
	-- SI EXISTE, LA ELIMINAMOS; SOLO PUEDE HABER UNA PORQUE ES PK
		RSP := 'RESULTADO: OK';
		SELECT id_proceso INTO vProceso FROM VULNERABILIDAD WHERE id_vulnerabilidad = pId;
		DELETE FROM VULNERABILIDAD WHERE id_vulnerabilidad = pId;
		-- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
		  -- Al dar de alta o de baja una vulnerabilidad (no se permite modificar el proceso ni el analisis asociados a la vulnerabilidad), se recalcula desde el mismo procedimiento ABM el número total de vulnerabilidades del proceso (num_vuln_detectadas)
		TFG.PKG_FUNCIONAMIENTO.ACTUALIZAR_VULNERABILIDADES_DE_PROCESO(vProceso,RSP);
		-- Registramos la actividad en el LOGOPERACIONES
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	ELSE 
	-- No existe
		RSP := 'RESULTADO: ERROR: La vulnerabilidad indicada no existe.';
		INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
		COMMIT;
	END IF;

   EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
	   COMMIT;
    WHEN OTHERS THEN
	   RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES (PROC,PAR_IN,RSP);
       COMMIT;
   END BAJA_VULNERABILIDAD;

PROCEDURE MODIFICA_VULNERABILIDAD ( 
   pId IN VULNERABILIDAD.id_vulnerabilidad%TYPE, 
   pF_deteccion IN VULNERABILIDAD.f_deteccion_vuln%TYPE,
   pEstado IN VULNERABILIDAD.id_estado_vuln%TYPE,
   pCritica IN VULNERABILIDAD.es_vuln_critica%TYPE,
   pNombre IN VULNERABILIDAD.nombre_vuln%TYPE,
   pURL IN VULNERABILIDAD.url_vuln%TYPE,
   RSP OUT VARCHAR2) AS
   
   MI_EXC   EXCEPTION;
   PAR_IN	VARCHAR2(1000);
   PROC     VARCHAR2(100);
   
   f_ini 	DATE;
   f_fin	DATE;         
   busca_pk 	NUMBER(1);
   vAnalisis	NUMBER(10);
   
  BEGIN
  PAR_IN := 'Vulnerabilidad: '||pId||', Fecha de deteccion: '||pF_deteccion||', ID estado: '||pEstado||', Es Critica: '||pCritica||', Nombre: '||pNombre||', URL Documentacion: '||pURL;
  PROC := 'PKG_ABM.MODIFICA_VULNERABILIDAD';
  
  -- No permitiremos cambiar el analisis y el proceso asociados a la vulnerabilidad, si el alta fue erronea habra que borrar la vulnerabilidad y volver a crearla nueva,
  
 -- NULOS: Comprobamos que ningun campo necesario tiene valor nulo.
  RSP := 'RESULTADO: ERROR: El identificador de la vulnerabilidad no puede ser nulo.';
  IF pId IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: La fecha de deteccion de la vulnerabilidad no puede ser nulo.';
  IF pF_deteccion IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El estado de la vulnerabilidad no puede ser nulo.';
  IF pEstado IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El indicador de si la vulnerabilidad es critica o no no puede ser nulo.';
  IF pCritica IS NULL THEN RAISE MI_EXC; END IF;
  RSP := 'RESULTADO: ERROR: El nombre de la vulnerabilidad no puede ser nulo.';
  IF pNombre IS NULL THEN RAISE MI_EXC; END IF;

  -- RESTRICCIONES: Si no hay valores necesarios nulos, comprobamos las restricciones de integridad que afecten.
  -- Los estados por los que puede pasar una vulnerabilidad son los registrados en la tabla ESTADOSVULN.
  -- Se podrian añadir nuevos estados, pero se inicializan al cargar las tablas maestras. 
  RSP := 'RESULTADO: ERROR: El estado indicado no existe.';
  SELECT COUNT(1) INTO busca_pk FROM ESTADOVULN WHERE id_estado_vuln = pEstado;
  IF busca_pk = 0 THEN RAISE MI_EXC; END IF;
  
  -- La fecha de detección de una vulnerabilidad siempre será anterior o igual a la fecha actual.
  RSP := 'RESULTADO: ERROR: La fecha de deteccion de la vulnerabilidad no puede estar en el futuro.';
  IF (pF_deteccion > SYSDATE) THEN RAISE MI_EXC; END IF;
  
  -- La fecha de deteccion de una vulnerabilidad deberá ser siempre posterior o igual a la fecha de inicio del analisis, y anterior o igual a la fecha de fin de dicho analisis (si se ha registrado).
  
  SELECT id_analisis INTO vAnalisis FROM VULNERABILIDAD WHERE id_vulnerabilidad = pId;
  
  SELECT f_inicio_analisis INTO f_ini FROM ANALISIS WHERE id_analisis = vAnalisis;
  SELECT f_fin_analisis INTO f_fin FROM ANALISIS WHERE id_analisis = vAnalisis;
  
  -- Como la fecha de fin puede o no estar registrada (no es obligatoria, ya que en principio se desconocera al principio del analisis), comprobamos la fecha de inicio, y si esta es correcta comparamos con la de fin, si existe.
  RSP := 'RESULTADO: ERROR: La fecha de la deteccion no puede ser anterior al inicio del analisis.';
  IF (pF_deteccion < f_ini) THEN RAISE MI_EXC; 
  ELSE 
	IF f_fin IS NOT NULL THEN
		RSP := 'RESULTADO: ERROR: La fecha del muestreo no puede ser posterior a la finalizacion del analisis.';
		IF (pF_deteccion > f_fin ) THEN RAISE MI_EXC; END IF;
	END IF;
  END IF;
  	
  -- MODIFICACION: Comprobado todo, tratamos de modificar la politica con los nuevos datos.
  -- Si alguno de los valores para un campo PK o UNIQUE esta repetido, saltara excepcion; si no salta, se insertara correctamente.
  RSP := 'RESULTADO: OK';
  UPDATE VULNERABILIDAD SET f_deteccion_vuln = pF_deteccion, id_estado_vuln =pEstado, es_vuln_critica = pCritica, nombre_vuln = pNombre, url_vuln = pURL  WHERE id_vulnerabilidad = pId;
  -- Registramos la actividad en el LOGOPERACIONES
  INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
    
  -- ESTADISTICAS: Actualizamos los campos estadisticos afectados por la ejecucion del procedimiento
  -- Nada en este caso.
  -- Al dar de alta o de baja una vulnerabilidad (no se permite modificar el proceso ni el analisis asociados a la vulnerabilidad), se recalcula desde el mismo procedimiento ABM el número total de vulnerabilidades del proceso (num_vuln_detectadas)
  
  -- ANOTAMOS LOS CAMBIOS
  COMMIT;
  
  -- Si ha saltado una excepcion, la procesamos
  EXCEPTION
    WHEN MI_EXC THEN
       INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
		COMMIT;
	-- Si se trata de insertar un valor duplicado, ya sea en una clave primaria, o en un campo definido como UNIQUE, controlamos la excepcion y la registramos
    WHEN DUP_VAL_ON_INDEX THEN
        RSP := 'RESULTADO: ERROR: Se esta tratando de insertar un valor duplicado en un campo que no lo permite.';
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
    WHEN OTHERS THEN 
        RSP := 'RESULTADO: ERROR: ' || TO_CHAR(SQLERRM(-SQLCODE));
        INSERT INTO LOGOPERACIONES (NOMBRE_PROCEDIMIENTO, PARAMS_ENTRADA, RESULTADO) VALUES(PROC,PAR_IN,RSP);
        COMMIT;
  END MODIFICA_VULNERABILIDAD; 
 
END PKG_ABM;